#!/usr/bin/env python

# flake8: noqa

import os
import json
import yaml
import json
import difflib

from configparser import ConfigParser

from nose.tools import *

from linchpin.InventoryFilters import GenericInventory

def setup_generic_inventory_filter():
    global filter
    global res_output

    filter = GenericInventory.GenericInventory()

    provider = 'general'
    base_path = '{0}'.format(os.path.dirname(
    os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))
    mock_path = '{0}/{1}/{2}'.format(lib_path, 'mockdata', provider)

    res_output = 'topo.json'
    res_file = open(mock_path+'/'+res_output)
    res_output = json.load(res_file)
    res_file.close()

def setup_generic_topology():
    global topology

    provider = 'dummy'
    base_path = '{0}'.format(os.path.dirname(
    os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))
    mock_path = '{0}/{1}/{2}'.format(lib_path, 'mockdata', provider)

    topo = 'topologies/dummy-cluster.yml'
    topo_file = open(mock_path+'/'+topo)
    topology = yaml.load(topo_file)
    topo_file.close()

def setup_generic_config():
    global config

    provider = 'general'
    base_path = '{0}'.format(os.path.dirname(
    os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))
    mock_path = '{0}/{1}/{2}'.format(lib_path, 'mockdata', provider)

    cfg = 'config.yml'
    cfg_file = open(mock_path+'/'+cfg)
    config = yaml.load(cfg_file)
    cfg_file.close()

def setup_generic_layout():
    global layout

    provider = 'layouts'
    base_path = '{0}'.format(os.path.dirname(
    os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))
    mock_path = '{0}/{1}/{2}'.format(lib_path, 'mockdata', provider)

    template = 'parsed-layout.json'
    template_file = open(mock_path+'/'+template)
    layout = json.load(template_file)

def setup_complex_workspace():
    global workspace

    provider = 'complex-inventory'
    base_path = '{0}'.format(os.path.dirname(
        os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))
    workspace = '{0}/{1}/{2}'.format(lib_path, 'mockdata', provider)


@with_setup(setup_generic_inventory_filter)
@with_setup(setup_generic_config)
def test_get_host_data():
    """
    """
    host_data = filter.get_host_data(res_output, config)
    expected_providers = ['aws', 'openstack', 'dummy', 'gcloud', 'beaker',
                          'libvirt', 'duffy', 'ovirt']
    assert_equal(set(host_data.keys()), set(expected_providers))

    for provider in host_data:
        for host in host_data[provider]:
            if provider == 'dummy' or provider == 'duffy':
                continue
            assert_true('__IP__' in host_data[provider][host].keys())


@with_setup(setup_generic_inventory_filter)
def test_get_hosts_by_count():
    """
    """
    host_dict = dict()
    host_dict['openstack'] = {}
    host_dict['openstack']['os1'] = 'one'
    host_dict['libvirt'] = {}
    host_dict['libvirt']['l1'] = 'one'
    host_dict['libvirt']['l2'] = 'two'
    expected_hosts = ['os1', 'l1', 'l2']

    hosts = filter.get_hosts_by_count(host_dict, 0, ['openstack', 'libvirt'])
    assert_equal(len(hosts), 0)
    hosts = filter.get_hosts_by_count(host_dict, 3, ['openstack', 'libvirt'])
    assert_equal(len(hosts), 3)
    assert_equal(set(hosts), set(expected_hosts))


@with_setup(setup_generic_inventory_filter)
@with_setup(setup_generic_topology)
@with_setup(setup_generic_config)
@with_setup(setup_generic_layout)
def test_get_inventory():
    """
    """
    inventory = filter.get_inventory(res_output, layout, topology, config)
    assert_true(inventory)

@with_setup(setup_complex_workspace)
@with_setup(setup_generic_inventory_filter)
@with_setup(setup_generic_config)
def test_output_order():
    """
    Test that inventories are ordered correctly

    This test checks that hosts and variables are ordered correctly when a
    provisioning output made up of multiple hosts is passed to the inventory
    generator.

    input: resources, topology, and layout from a mock succcessful provisioning
    output: an inventory file whose order will be verified
    """

    # get topology and layout
    pf_path = '{0}/{1}'.format(workspace, 'PinFile')
    pinfile = yaml.load(open(pf_path))
    topology = pinfile['complex-inventory']['topology']
    layout_path = '{0}/{1}'.format(workspace, 'layout.json')
    layout = json.load(open(layout_path))

    # get res_output
    output_path = '{0}/{1}'.format(workspace, 'linchpin.benchmark')
    res_output = json.load(open(output_path))
    res_output = res_output[res_output.keys()[0]]['targets'][0]['complex-inventory']['outputs']['resources']

    # call get_inventory and print the result
    inventory = filter.get_inventory(res_output, layout, topology, config)

    # load in "correct" inventory file
    correct_inv_path = '{0}/{1}'.format(workspace,
                                            'correct-inventory')
    correct_inventory = open(correct_inv_path, 'r').read()

    # check that the two inventories are equal
    inventory_lines = inventory.splitlines(1)
    correct_lines = correct_inventory.splitlines(1)
    # if the assertion fails, this diff will display
    diff = difflib.unified_diff(inventory_lines, correct_lines)
    print ''.join(diff)
    assert_equal(inventory, correct_inventory)



#!/usr/bin/env python

# flake8: noqa

from nose.tools import assert_equal

import linchpin.InventoryFilters
from linchpin.InventoryFilters import InventoryProviders

from linchpin.InventoryFilters.AWSInventory import AWSInventory
from linchpin.InventoryFilters.BeakerInventory import BeakerInventory
from linchpin.InventoryFilters.DuffyInventory import DuffyInventory
from linchpin.InventoryFilters.DummyInventory import DummyInventory
from linchpin.InventoryFilters.GCloudInventory import GCloudInventory
from linchpin.InventoryFilters.LibvirtInventory import LibvirtInventory
from linchpin.InventoryFilters.OpenstackInventory import OpenstackInventory
from linchpin.InventoryFilters.OvirtInventory import OvirtInventory

def test_get_driver():
    driver = InventoryProviders.get_driver("aws")
    assert_equal(driver, AWSInventory)

def test_get_all_drivers():
    drivers = {
        "aws": AWSInventory,
        "beaker": BeakerInventory,
        "duffy": DuffyInventory,
        "dummy": DummyInventory,
        "gcloud": GCloudInventory,
        "libvirt": LibvirtInventory,
        "openstack": OpenstackInventory,
        "ovirt": OvirtInventory,
    }
    if_drivers = InventoryProviders.get_all_drivers()
    assert_equal(drivers,if_drivers)


import os
import sys
import time
import shutil
import tempfile

from nose.tools import assert_equal
from nose.tools import assert_list_equal
from nose.tools import with_setup

from linchpin.cli import LinchpinCli
from linchpin.cli.context import LinchpinCliContext
from linchpin.tests.mockdata.contextdata import ContextData


def test_context_create():

    lpc = LinchpinCliContext()
    assert_equal(isinstance(lpc, LinchpinCliContext), True)


def setup_load_config():

    """
    Perform setup of ContextData() object, and run get_temp_filename()
    """

    global cd
    global provider
    global config_path
    global config_data

    provider = 'dummy'

    cd = ContextData()
    # cd.load_config_data(provider)
    cd.load_config_data()
    cd.create_config()
    config_path = cd.get_temp_filename()
    config_data = cd.cfg_data
    cd.write_config_file(config_path)


def setup_lp_fetch_env():
    global lpc
    global lpcli

    global target
    global pinfile
    global mockpath

    base_path = '{0}'.format(os.path.dirname(
        os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))

    setup_load_config()

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.load_global_evars()
    lpc.setup_logging()

    lpc.workspace = tempfile.mkdtemp(prefix='fetch_')
    print('workspace: {0}'.format(lpc.workspace))

    lpcli = LinchpinCli(lpc)


@with_setup(setup_lp_fetch_env)
def test_fetch_git():

    shutil.rmtree(lpc.workspace)
    os.makedirs(lpc.workspace)
    os.chdir(lpc.workspace)
    src_url = 'git://github.com/herlo/lp_test_workspaces'
    root_dir = 'os-server-addl-vols'

    lpcli.lp_fetch(src_url, root=root_dir)

    chk_dir = '{0}/{1}/'.format(lpc.workspace, root_dir)
    assert(os.path.exists('{0}/PinFile'.format(chk_dir)))


@with_setup(setup_lp_fetch_env)
def test_fetch_http():

    shutil.rmtree(lpc.workspace)
    os.makedirs(lpc.workspace)
    os.chdir(lpc.workspace)
    src_url = 'https://herlo.fedorapeople.org'
    root_dir = 'simple'

    lpcli.lp_fetch(src_url, root='lp_ws/{0}'.format(root_dir),
                   fetch_protocol='FetchHttp')

    chk_dir = '{0}/{1}/'.format(lpc.workspace, root_dir)
    assert(os.path.exists('{0}/PinFile'.format(chk_dir)))


def main():

    setup_load_config()


if __name__ == '__main__':
    sys.exit(main())



import os
import subprocess
from .action_manager import ActionManager
from cerberus import Validator

from linchpin.exceptions import HookError


class SubprocessActionManager(ActionManager):

    def __init__(self, name, action_data, target_data, **kwargs):

        """
        SubprocessActionManager constructor
        :param name: Name of Action Manager , ( ie., shell)
        :param action_data: dictionary of action_block
        consists of set of actions
        example:
        - name: hookname
          type: shell
          actions:
            - echo " this is post down operation Hello hai how r u ?"
            - test.sh
        :param target_data: Target specific data defined in PinFile
        :param kwargs: anyother keyword args passed as metadata
        """

        self.name = name
        self.action_data = action_data
        self.target_data = target_data
        self.context = kwargs.get('context', True)
        self.kwargs = kwargs


    def validate(self):

        """
        Validates the action_block based on the cerberus schema
        example:
        action_block :: sample ::
        - name: manipulate_inventory
          type: shell
          path: /tmp/shellscripts
          actions:
            - thisisshell.sh
        """

        schema = {
            'name': {'type': 'string', 'required': True},
            'type': {
                'type': 'string',
                'allowed': ['shell', 'subprocess']
            },
            'path': {'type': 'string', 'required': False},
            'context': {'type': 'boolean', 'required': False},
            'actions': {
                'type': 'list',
                'schema': {'type': 'string'},
                'required': True
            }
        }

        v = Validator(schema)
        status = v.validate(self.action_data)

        if not status:
            raise HookError("Invalid Syntax: {0}".format(str(v.errors)))
        else:
            return status


    def load(self):

        """
        Adds the shell script to the os path if mentioned
        """

        # set os.environpath if exists
        if 'path' in self.action_data:
            os.environ["PATH"] += ":{0}".format(self.action_data["path"])


    def add_context_params(self, action):

        """
        Adds ctx params to the action_block run when context is true
        :param file_path: path to the script
        :param context: whether the context params are to be included or not
        """

        command = action
        for key in self.target_data:
            command += " {0}={1} ".format(key, self.target_data[key])
        return command


    def execute(self):

        """
        Executes the action_block in the PinFile
        """

        self.load()
        for action in self.action_data["actions"]:
            if self.context:
                command = self.add_context_params(action)
            else:
                command = action
            proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
            proc.wait()
            for line in proc.stdout:
                print(line)



from abc import ABCMeta, abstractmethod


class RunDB(object):

    __metaclass__ = ABCMeta

    @abstractmethod
    def init_table(self, table):
        return self.driver.init_table(table)

    @abstractmethod
    def update_record(self, table, run_id, key, value):
        return self.driver.update_record(table, run_id, key, value)

    @abstractmethod
    def get_tx_record(self, tx_id):
        return self.driver.get_tx_record(tx_id)

    def get_tx_records(self, tx_ids):
        return self.driver.get_tx_records(tx_ids)

    @abstractmethod
    def get_record(self, table, action='up', run_id=None):
        return self.driver.get_record(table, run_id=run_id)

    @abstractmethod
    def get_records(self, table=[], count=10):
        return self.driver.get_records(table=table, count=count)

    @abstractmethod
    def get_tables(self):
        return self.driver.get_tables()

    @abstractmethod
    def remove_record(self, table, key, value):
        return self.driver.remove_record(table, key, value)

    @abstractmethod
    def search(self, table, key):
        return self.driver.search(table, key)

    @abstractmethod
    def query(self, table, query_info):
        return self.driver.query(table, query_info)

    @abstractmethod
    def purge(self, table=None):
        return self.driver.purge(table)



#!/usr/bin/env python


def merge_two_dicts(x, y):
    z = x.copy()   # start with x's keys and values
    z.update(y)    # modifies z with y's keys and values & returns None
    return z


def combine_hosts_names(hosts, names):
    result = []
    for i in range(len(names)):
        result.append(merge_two_dicts(hosts[i], names[i]))
    return result


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'combine_hosts_names': combine_hosts_names
        }



# -*- coding: utf-8 -*-

# """
#    click_default_group
#    ~~~~~~~~~~~~~~~~~~~
#    Define a default subcommand by `default=True`:
#    .. sourcecode:: python
#       import click
#       from click_default_group import DefaultGroup
#       @click.group(cls=DefaultGroup, default_if_no_args=True)
#       def cli():
#           pass
#       @cli.command(default=True)
#       def foo():
#           click.echo('foo')
#       @cli.command()
#       def bar():
#           click.echo('bar')
#    Then you can invoke that without explicit subcommand name:
#    .. sourcecode:: console
#       $ cli.py --help
#       Usage: cli.py [OPTIONS] COMMAND [ARGS]...
#       Options:
#         --help    Show this message and exit.
#       Command:
#         foo*
#         bar
#       $ cli.py
#       foo
#       $ cli.py foo
#       foo
#       $ cli.py bar
#       bar
#
# Credit: Heungsub Lee <https://github.com/sublee>
#
# Copyright (c) 2015-2016, Heungsub Lee
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification,are permitted provided that the following conditions are met:
#
#   Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# """

import warnings
import click


__all__ = ['DefaultGroup']
__version__ = '1.2'


class DefaultGroup(click.Group):
    """
    Invokes a subcommand marked with `default=True` if any subcommand not
    chosen.

    :param default_if_no_args: resolves to the default command if no arguments
                               passed.
    """

    def __init__(self, *args, **kwargs):
        # To resolve as the default command.
        if not kwargs.get('ignore_unknown_options', True):
            raise ValueError('Default group accepts unknown options')
        self.ignore_unknown_options = True
        self.default_cmd_name = kwargs.pop('default', None)
        self.default_if_no_args = kwargs.pop('default_if_no_args', False)
        super(DefaultGroup, self).__init__(*args, **kwargs)


    def set_default_command(self, command):
        """Sets a command function as the default command."""
        cmd_name = command.name
        self.add_command(command)
        self.default_cmd_name = cmd_name


    def parse_args(self, ctx, args):

        super(DefaultGroup, self).parse_args(ctx, args)

        # modified from original code to allow default help command
        # when parameters are passed in.
        if not len(ctx.protected_args):
            ctx.protected_args.append(self.default_cmd_name)

    def list_commands(self, ctx):
        """
        Provide a list of available commands. Anything deprecated should
        not be listed
        """

        return ['init', 'up', 'destroy', 'fetch', 'journal']

    def get_command(self, ctx, cmd_name):

        lp_aliases = {
            'rise': 'up',
            'drop': 'destroy',
            'down': 'destroy',
        }

        cmd = lp_aliases.get(cmd_name, cmd_name)

        if cmd not in self.commands:
            # No command name matched.
            ctx.arg0 = cmd
            cmd = self.default_cmd_name
        return super(DefaultGroup, self).get_command(ctx, cmd)


    def resolve_command(self, ctx, args):
        base = super(DefaultGroup, self)
        cmd_name, cmd, args = base.resolve_command(ctx, args)
        if hasattr(ctx, 'arg0'):
            args.insert(0, ctx.arg0)
        return cmd_name, cmd, args


    def format_commands(self, ctx, formatter):
        formatter = DefaultCommandFormatter(self, formatter, mark='*')
        return super(DefaultGroup, self).format_commands(ctx, formatter)


    def command(self, *args, **kwargs):
        default = kwargs.pop('default', False)
        decorator = super(DefaultGroup, self).command(*args, **kwargs)
        if not default:
            return decorator
        warnings.warn('Use default param of DefaultGroup or '
                      'set_default_command() instead', DeprecationWarning)


        def _decorator(f):
            cmd = decorator(f)
            self.set_default_command(cmd)
            return cmd
        return _decorator


class DefaultCommandFormatter(object):
    """Wraps a formatter to mark a default command."""


    def __init__(self, group, formatter, mark='*'):
        self.group = group
        self.formatter = formatter
        self.mark = mark


    def __getattr__(self, attr):
        return getattr(self.formatter, attr)


    def write_dl(self, rows, *args, **kwargs):
        rows_ = []
        for cmd_name, help in rows:
            if cmd_name == self.group.default_cmd_name:
                rows_.insert(0, (cmd_name + self.mark, help))
            else:
                rows_.append((cmd_name, help))
        return self.formatter.write_dl(rows_, *args, **kwargs)



from abc import ABCMeta, abstractmethod

# action manager abstract class to be implemented by all
# action managers


class ActionManager(object):

    __metaclass__ = ABCMeta

    @abstractmethod
    def validate(self):
        pass

    @abstractmethod
    def execute(self):
        pass



#!/usr/bin/env python

import abc


class InventoryFormatter:
    __metaclass__ = abc.ABCMeta

    def __init__(self):
        pass

    @abc.abstractmethod
    def add_sections(self, section_list):
        pass

    @abc.abstractmethod
    def set_children(self, inv):
        pass

    @abc.abstractmethod
    def set_vars(self, inv):
        pass

    @abc.abstractmethod
    def add_ips_to_groups(self, inven_hosts, layout):
        pass

    @abc.abstractmethod
    def add_common_vars(self, host_groups, layout):
        pass

    @abc.abstractmethod
    def generate_inventory(self):
        pass



from .action_manager import ActionManager
from Naked.toolshed.shell import run_rb
from cerberus import Validator

from linchpin.exceptions import HookError


class RubyActionManager(ActionManager):

    def __init__(self, name, action_data, target_data, **kwargs):

        """
        RubyActionManager constructor
        :param name: Name of Action Manager , ( ie., ruby)
        :param action_data: dictionary of action_block consists of
        a set of actions
        example:
        - name: nameofhook
          type: ruby
          context: true
          actions:
            - test.js
        :param target_data: Target specific data defined in PinFile
        :param kwargs: anyother keyword args passed as metadata
        """

        self.name = name
        self.action_data = action_data
        self.target_data = target_data
        self.context = kwargs.get('context', True)
        self.kwargs = kwargs


    def validate(self):

        """
        Validates the action_block based on the cerberus schema
        """

        schema = {
            'name': {'type': 'string', 'required': True},
            'type': {'type': 'string', 'allowed': ['ruby']},
            'path': {'type': 'string', 'required': False},
            'context': {'type': 'boolean', 'required': False},
            'actions': {
                'type': 'list',
                'schema': {'type': 'string'},
                'required': True
            }
        }

        v = Validator(schema)
        status = v.validate(self.action_data)

        if not status:
            raise HookError("Invalid syntax: {0}".format(str((v.errors))))
        else:
            return status


    def add_ctx_params(self, file_path, context=True):

        """
        Adds ctx params to the action_block run when context is true
        :param file_path: path to the script
        :param context: whether the context params are to be included or not
        """

        if not context:
            return file_path

        params = file_path
        for key in self.target_data:
            params += " {0}={1} ".format(key, self.target_data[key])

        return "{0} {1}".format(file_path, params)


    def execute(self):

        """
        Executes the action_block in the PinFile
        """

        for action in self.action_data["actions"]:
            context = self.action_data.get("context", True)
            path = self.action_data["path"]
            file_path = "{0}/{1}".format(
                        path,
                        action
            )

            command = self.add_ctx_params(file_path, context)
            success = run_rb(command)
            return success



#!/usr/bin/env python

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from .InventoryFilter import InventoryFilter


class AWSInventory(InventoryFilter):
    DEFAULT_HOSTNAMES = ['public_dns_name', 'public_ip', 'private_ip']

    def get_host_data(self, topo, cfgs):
        """
        Returns a dict of hostnames or IP addresses for use in an Ansible
        inventory file, based on available data. Only a single hostname or IP
        address will be returned per instance, so as to avoid duplicate runs of
        Ansible on the same host via the generated inventory file.

        Each hostname contains mappings of any variable that was defined in the
        cfgs section of the PinFile (e.g. __IP__) to the value in the field that
        corresponds with that variable in the cfgs.

        If an instance has a public IP attached, its hostname in DNS will be
        returned if available, and if not the public IP address will be used.
        For instances which have a private IP address for VPC use cases, the
        private IP address will be returned since private EC2 hostnames (e.g.
        ip-10-0-0-1.ec2.internal) will not typically be resolvable outside of
        AWS. For instances with both a public and private IP address, the
        public address is always returned instead of the private address.

        :param topo:
            linchpin AWS EC2 resource data

        :param cfgs:
            map of config options from PinFile
        """

        host_data = {}
        var_data = cfgs.get('aws', {})
        if var_data is None:
            var_data = {}
        for group in topo.get('aws_ec2_res', []):
            for instance in group['instances']:
                host = self.get_hostname(instance, var_data,
                                         self.DEFAULT_HOSTNAMES)
                hostname_var = host[0]
                hostname = host[1]
                if '__IP__' not in var_data.keys():
                    var_data['__IP__'] = hostname_var
                host_data[hostname] = {}
                self.set_config_values(host_data[hostname], instance, var_data)
        return host_data

    def get_host_ips(self, host_data):
        return host_data.keys()

    def get_inventory(self, topo, layout, config):
        if len(topo['aws_ec2_res']) == 0:
            return ""
        host_data = self.get_host_data(topo, config)
        inven_hosts = self.get_host_ips(host_data)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO()
        self.config.write(output)
        return output.getvalue()



from ansible.plugins.callback import CallbackBase


class PlaybookCallback(CallbackBase):

    """Playbook callback"""


    def __init__(self, display=None, options=None, ansible_version=2.3):

        # note the following if else ladder should be restructured after
        # linchpin ansible minimum requirements changed until then
        # code should be remained unchanged to maintain backward
        # compatibility to ansible 2.3.1 version
        if ansible_version >= 2.5:
            self._load_name = None
            super(PlaybookCallback, self).__init__(display=display,
                                                   options=options)
        elif ansible_version >= 2.4:
            super(PlaybookCallback, self).__init__(display=display,
                                                   options=options)
        else:
            # since ansible 2.3.1 version does not support options in
            # inside PlaybbookCallback the options are omitted
            super(PlaybookCallback, self).__init__(display=display)

        self._options = options
        self._display.verbosity = options.verbosity

        # store all results
        self.results = {'failed': [], 'ok': []}


    def v2_runner_on_ok(self, result):

        """Save ok result"""

        self.results['ok'].append(result)

    def v2_runner_on_failed(self, result, **kwargs):

        """Save failed result"""

        ignore_errors = kwargs.get('ignore_errors')

        if not ignore_errors:
            self.results['failed'].append(result)



#!/usr/bin/env python

# flake8: noqa

import os
import json
import yaml

from nose.tools import *

from linchpin.InventoryFilters import DummyInventory

def setup_dummy_inventory_filter():
    global filter
    global topo

    filter = DummyInventory.DummyInventory()
    
    provider = 'general'
    base_path = '{0}'.format(os.path.dirname(
    os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))
    mock_path = '{0}/{1}/{2}'.format(lib_path, 'mockdata', provider)

    topology = 'topo.json'
    topo_file = open(mock_path+'/'+topology)
    topo = json.load(topo_file)
    topo_file.close()

def setup_dummy_config():
    global config

    provider = 'general'
    base_path = '{0}'.format(os.path.dirname(
    os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))
    mock_path = '{0}/{1}/{2}'.format(lib_path, 'mockdata', provider)

    cfg = 'config.yml'
    cfg_file = open(mock_path+'/'+cfg)
    config = yaml.load(cfg_file)
    cfg_file.close()

def setup_dummy_layout():
    global layout

    provider = 'layouts'
    base_path = '{0}'.format(os.path.dirname(
    os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))
    mock_path = '{0}/{1}/{2}'.format(lib_path, 'mockdata', provider)

    template = 'parsed-layout.json'
    template_file = open(mock_path+'/'+template)
    layout = json.load(template_file)

@with_setup(setup_dummy_inventory_filter)
@with_setup(setup_dummy_config)
def test_get_host_data():
    """
    """
    host_data = filter.get_host_data(topo, config)
    expected_vars = ['__IP__']
    for host in host_data:
        assert_equal(set(host_data[host].keys()), set(expected_vars))

@with_setup(setup_dummy_inventory_filter)
@with_setup(setup_dummy_config)
def test_get_host_ips():
    """
    """
    host_data = filter.get_host_data(topo, config)
    ips = filter.get_host_ips(host_data)
    expected_hosts = ["web-b3a49a-0.example.net", "web-b3a49a-1.example.net",
                      "web-b3a49a-2.example.net", "test-b3a49a-0.example.net"]
    assert_equal(set(host_data.keys()), set(expected_hosts))

@with_setup(setup_dummy_inventory_filter)
@with_setup(setup_dummy_config)
@with_setup(setup_dummy_layout)
def test_get_inventory():
    """
    """
    empty_topo = dict()
    empty_topo['dummy_res'] = []
    inventory = filter.get_inventory(empty_topo, layout, config)
    assert_false(inventory)
    inventory = filter.get_inventory(topo, layout, config)
    assert_true(inventory)


from .subprocess_action_manager import SubprocessActionManager
from .ansible_action_manager import AnsibleActionManager
from .python_action_manager import PythonActionManager
from .ruby_action_manager import RubyActionManager
from .nodejs_action_manager import NodejsActionManager

# dict of core action manager packaged with linchpin

ACTION_MANAGERS = {
    "shell": SubprocessActionManager,
    "ansible": AnsibleActionManager,
    "python": PythonActionManager,
    "ruby": RubyActionManager,
    "nodejs": NodejsActionManager
}



#!/usr/bin/env python

try:
    from StrinIO import StringIO
except ImportError:
    from io import StringIO

import collections

from .InventoryFormatter import InventoryFormatter

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser


class CFGInventoryFormatter(InventoryFormatter):

    def __init__(self):
        InventoryFormatter.__init__(self)
        self.config = ConfigParser(allow_no_value=True)

    def add_sections(self, section_list):
        for section in section_list:
            self.config.add_section(section)
        # adding a default section all
        if "all" not in self.config.sections():
            self.config.add_section("all")

    def set_children(self, inv):
        if 'host_groups' not in inv.keys():
            return
        for host_group in inv['host_groups']:
            if "children" in inv['host_groups'][host_group]:
                self.config.add_section("{0}:children".format(host_group))
                for child in inv['host_groups'][host_group]['children']:
                    self.config.set("{0}:children".format(host_group), child)

    def set_vars(self, inv):
        if 'host_groups' not in inv.keys():
            return
        for host_group in inv['host_groups']:
            if "vars" in inv['host_groups'][host_group]:
                self.config.add_section("{0}:vars".format(host_group))
                for var in inv['host_groups'][host_group]['vars']:
                    grp_vars = inv['host_groups'][host_group]['vars'][var]
                    grp_vars = str(grp_vars)
                    self.config.set(host_group + ":" + "vars", var, grp_vars)

    def add_ips_to_groups(self, inven_hosts, layout):
        # create a ip to host mapping based on count
        ip_to_host = collections.OrderedDict()
        inven_hosts.reverse()
        for host_name in layout['hosts']:
            if 'count' in host_name.keys():
                count = host_name['count']
            else:
                count = 1
            host_list = []
            if count > len(inven_hosts):
                count = len(inven_hosts)
            for i in range(count, 0, -1):
                item = inven_hosts.pop()
                host_list.append(item)
            ip_to_host[host_name["name"]] = host_list
        # add ips to the respective host groups in inventory
        for host_name in layout['hosts']:
            host_ips = ip_to_host[host_name["name"]]
            for ip in host_ips:
                for host_group in host_name['host_groups']:
                    self.config.set(host_group, ip)
                    self.config.set("all", ip)

    def add_common_vars(self, host_groups, layout, config):
        # defaults common_vars to [] when they doesnot exist
        host_groups.append("all")
        common_vars = layout['vars'] if 'vars' in layout.keys() else []
        for group in host_groups:
            items = collections.OrderedDict(self.config.items(group)).keys()
            self.config.remove_section(group)
            self.config.add_section(group)
            for item in items:
                host_string = item
                for var in common_vars:
                    for provider in config.keys():
                        if item not in config[provider].keys():
                            continue
                        if common_vars[var] in config[provider][item].keys():
                            value = common_vars[var]
                            host_string += " " + var + "=" +\
                                           config[provider][item][value]
                        else:
                            host_string += " " + var + "=" + common_vars[var]
                self.config.set(group, host_string)

    def generate_inventory(self):
        output = StringIO()
        self.config.write(output)
        return output.getvalue()



import os

import tempfile
import ConfigParser

from linchpin.exceptions import LinchpinError

"""
Provide valid context data to test against.

"""


class ContextData(object):

    def __init__(self, parser=ConfigParser.SafeConfigParser):


        self.lib_path = '{0}'.format(os.path.dirname(
            os.path.realpath(__file__))).rstrip('/')

        current_path = os.path.dirname(os.path.realpath(__file__))
        constants_path = '{0}/../../'.format(current_path)

        self.constants_path = '{0}'.format(os.path.dirname(
            constants_path)).rstrip('/')

        self.logfile = tempfile.mktemp(suffix='.log', prefix='linchpin')
        self.parser = parser()

        self.cfg_data = {}
        self._load_constants()


    def _load_constants(self):
        """
        Create self.cfgs with defaults from the linchpin constants file.
        """

        constants_file = '{0}/linchpin.constants'.format(self.constants_path)
        constants_file = os.path.realpath(os.path.expanduser(constants_file))
        self._parse_config(constants_file)


    def load_config_data(self, provider='dummy'):
        """
        Load a test-based linchpin.conf into both a configs and evars
        dictionary to represent a configuration file

        """


        expanded_path = None
        config_found = False

        # simply modify this variable to adjust where linchpin.conf can be found
        CONFIG_PATH = [
            '{0}/{1}/conf/linchpin.conf'.format(self.lib_path, provider)
        ]

        for path in CONFIG_PATH:
            expanded_path = (
                "{0}".format(os.path.realpath(os.path.expanduser(path))))

            if os.path.exists(expanded_path):
                self._parse_config(expanded_path)


        # override logger file
        self.cfg_data['logger'] = dict()
        self.cfg_data['logger']['file'] = self.logfile

        self.evars = self.cfg_data.get('evars', {})


    def _parse_config(self, path):
        """
        Parse configs into the self.cfg_data dict from provided path.

        :param path: A path to a config to parse
        """

        try:
            config = ConfigParser.SafeConfigParser()
            f = open(path)
            config.readfp(f)
            f.close()

            for section in config.sections():
                if not self.cfg_data.get(section):
                    self.cfg_data[section] = {}

                for k in config.options(section):
                    if section == 'evars':
                        try:
                            self.cfg_data[section][k] = (
                                config.getboolean(section, k)
                            )
                        except ValueError:
                            self.cfg_data[section][k] = config.get(section, k)
                    else:
                        try:
                            self.cfg_data[section][k] = config.get(section, k)
                        except ConfigParser.InterpolationMissingOptionError:
                            value = config.get(section, k, raw=True)
                            self.cfg_data[section][k] = value.replace('%%', '%')

        except ConfigParser.InterpolationSyntaxError as e:
            raise LinchpinError('Unable to parse configuration file properly:'
                                ' {0}'.format(e))


    def get_temp_filename(self):

        tmpfile = tempfile.NamedTemporaryFile(delete=False).name
        return tmpfile


    def write_config_file(self, path):

        try:
            with open(path, 'a+') as f:
                self.parser.write(f)
        except Exception as e:
            raise LinchpinError('Unable to write configuration file:'
                                ' {0}'.format(e))


    def create_config(self, config_data=None):

        """
        Creates a config object using ConfigParser from the config_data object

        """

        if not config_data:
            config_data = self.cfg_data

        # we know that data is a dict, containing dicts
        try:
            for k, v in config_data.iteritems():
                self.parser.add_section(k)
                for kv, vv in v.iteritems():
                    if type(vv) is not str:
                        vv = str(vv)
                    self.parser.set(k, kv, vv)
        except ValueError:
            pass



#!/usr/bin/env python


def get_pod_status(pod_list, res_def_out):
    # this filter runs only when the kind is "Pod"
    pod_status = pod_list.get("resources", [])
    container_names = [x["name"]
                       for x in res_def_out["result"]["spec"]["containers"]]
    pod_stats = {}
    # filters out all the pods data w.r.to containers
    # provisioned in pods
    for x in pod_status:
        if x["metadata"]["name"] in container_names:
            pod_data = x
            pod_data["status"] = x["status"]["phase"]
            pod_stats[x["metadata"]["name"]] = pod_data
    # checks for the status of the container and returns Failure value
    # when any container is not in Running state
    for name in container_names:
        if pod_stats[name]["status"] != "Running":
            return "Failure"
    return pod_stats


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'get_pod_status': get_pod_status
        }



#!/usr/bin/env python
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from .InventoryFilter import InventoryFilter


class OpenstackInventory(InventoryFilter):
    DEFAULT_HOSTNAMES = ["accessIPv4", "public_v4", "private_v4"]

    def get_host_data(self, topo, cfgs):
        host_data = {}
        var_data = cfgs.get('openstack', {})
        if var_data is None:
            var_data = {}

        for group in topo['os_server_res']:
            if 'results' in group.keys():
                for res in group.get('results', []):
                    if 'openstack' in res.keys():
                        os_vars = res.get('openstack', [])
                        host = self.get_hostname(os_vars, var_data,
                                                 self.DEFAULT_HOSTNAMES)
                        hostname_var = host[0]
                        hostname = host[1]
                        host_data[hostname] = {}
                        if '__IP__' not in var_data.keys():
                            var_data['__IP__'] = hostname_var
                            host_data[hostname] = {}
                        self.set_config_values(host_data[hostname], os_vars,
                                               var_data)
                    else:
                        continue
            else:
                grp = group.get('openstack', [])
                if isinstance(grp, list):
                    for server in grp:
                        host = self.get_hostname(server, var_data,
                                                 self.DEFAULT_HOSTNAMES)
                        hostname_var = host[0]
                        hostname = host[1]
                        host_data[hostname] = {}
                        if '__IP__' not in var_data.keys():
                            var_data['__IP__'] = hostname_var
                            host_data[hostname] = {}
                        host_data[hostname] = {}
                        self.set_config_values(host_data[hostname], server,
                                               var_data)
                if isinstance(grp, dict):
                    host = self.get_hostname(grp, var_data,
                                             self.DEFAULT_HOSTNAMES)
                    hostname_var = host[0]
                    hostname = host[1]
                    host_data[hostname] = {}
                    if '__IP__' not in var_data.keys():
                        var_data['__IP__'] = hostname_var
                        host_data[hostname] = {}
                    host_data[hostname] = {}
                    self.set_config_values(host_data[hostname], grp, var_data)
        return host_data


    def get_host_ips(self, host_data):
        return host_data.keys()

    def get_inventory(self, topo, layout, config):
        if len(topo['os_server_res']) == 0:
            return ""
        host_data = self.get_host_data(topo, config)
        inven_hosts = self.get_host_ips(host_data)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO()
        self.config.write(output)
        return output.getvalue()



#!/usr/bin/env python


def get_host_from_uri(uri):
    """
    examples: qemu+ssh://root@hail.cloud.example.com/system
              test:///default
              qemu+ssh://192.168.122.6/system
    """
    uri = uri.split("//")[-1].split("/")[0].split("@")[-1]
    if uri == '':
        return 'localhost'
    return uri


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'get_host_from_uri': get_host_from_uri
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author: Samvaran Kashyap Rallabandi -  <srallaba@redhat.com>
#
# Auth driver  for Ansible based infra provsioning tool, linch-pin

import os
import yaml
import json
import ansible

from ansible.module_utils.basic import AnsibleModule

try:
    from ansible.parsing.vault import VaultLib
except ImportError:
    # Ansible<2.0
    from ansible.utils.vault import VaultLib

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser as ConfigParser


_ansible_ver = float('.'.join(ansible.__version__.split('.')[:2]))

DOCUMENTATION = '''
---
version_added: "0.1"
module: auth_driver
short_description: auth_driver module in ansible
description:
  - This module allows a user to fetch credentials on request and register it
    as variable in ansible.

options:
  name:
    description:
      name of the credential file to be used
    required: true
  cred_type:
    description:
      credential type , type of credential to be used.
      eg: aws, gcloud , openstack , etc.,
    required: false
  cred_path:
    description:
      credentials path where the credentials are to be stored
    required: false
  driver:
    description:
      defaults to file type.
    required: true

'''


def _make_secrets(secret):
    if _ansible_ver < 2.4:
        return secret
    from ansible.constants import DEFAULT_VAULT_ID_MATCH
    from ansible.parsing.vault import VaultSecret
    return [(DEFAULT_VAULT_ID_MATCH, VaultSecret(secret))]


class ConfigDict(ConfigParser.ConfigParser):

    def as_dict(self):
        d = dict(self._sections)
        for k in d:
            d[k] = dict(self._defaults, **d[k])
            d[k].pop('__name__', None)
        return d


def parse_file(filename, vault_enc, vault_pass):
    if json.loads(vault_enc.lower()):
        vault_pass = vault_pass.encode('utf-8')
        vault = VaultLib(_make_secrets(vault_pass))
        cred_str = vault.decrypt(open(filename).read())
    else:
        cred_str = open(filename, "r").read()
    try:
        out = json.loads(cred_str)
    except Exception as e:
        try:
            out = yaml.load(cred_str)
        except Exception as e:
            try:
                config = ConfigDict()
                f = open(filename)
                config.readfp(f)
                out = config.as_dict()
                f.close()
            except Exception as e:
                module.fail_json(msg="Error: {0} ".format(str(e)))
    return out


def get_cred(fname, creds_path, vault_enc, vault_pass):

    paths = creds_path.split(os.path.pathsep)
    # files = []
    for path in paths:
        path = os.path.realpath(os.path.expanduser(path))
        for filename in os.listdir(path):
            if fname == filename:
                full_file_path = '{0}/{1}'.format(path, filename)
                out = parse_file(full_file_path, vault_enc, vault_pass)
                return out, path

    module.fail_json(msg="Error: Credential not found")


def main():

    global module
    module = AnsibleModule(
        argument_spec={
            'filename': {'required': True, 'aliases': ['name']},
            'cred_type': {'required': False, 'aliases': ['credential_type']},
            'cred_path': {'required': True, 'aliases': ['credential_store']},
            'driver': {'default': 'file', 'aliases': ['driver_type']},
            'vault_pass': {'default': '', 'aliases': ['vault_pass',
                                                      'vault_password']},
            'vault_enc': {'default': False, 'aliases': ['vault_encryption']},
        },
        required_one_of=[],
        supports_check_mode=True
    )
    filename = module.params["filename"]
    cred_type = module.params["cred_type"]  # noqa F841
    cred_path = module.params["cred_path"]
    driver_type = module.params["driver"]  # noqa F841
    vault_pass = module.params["vault_pass"]  # noqa F841
    vault_enc = module.params["vault_enc"]  # noqa F841
    output, path = get_cred(filename, cred_path, vault_enc, vault_pass)
    changed = True
    module.exit_json(changed=changed,
                     output=output, params=module.params, path=path)


main()



#!/usr/bin/env python

# flake8: noqa

import os

from nose.tools import *

import logging
from unittest import TestCase

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser as ConfigParser

from linchpin.cli.context import LinchpinCliContext
from linchpin.tests.mockdata.contextdata import ContextData


def test_context_create():

    lpc = LinchpinCliContext()
    assert_equal(isinstance(lpc, LinchpinCliContext), True)


def setup_context_data():

    """
    Perform setup of ContextData() object, and run get_temp_filename()
    """

    global config_path
    global config_data
    global evars_data
    global logfile

    cd = ContextData()
    cd.load_config_data()
    cd.create_config()
    evars_data = cd.evars
    config_path = cd.get_temp_filename()
    config_data = cd.cfg_data

    logfile = cd.logfile
    cd.write_config_file(config_path)


@with_setup(setup_context_data)
def test_load_config():

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    cfg_data = dict(config_data)

    # remove things that will always differ
    for cfg in ['lp', 'evars']:
        lpc.cfgs.pop(cfg)
        cfg_data.pop(cfg)

    assert_dict_equal(cfg_data['console'],
                      lpc.cfgs['console'])


@with_setup(setup_context_data)
def test_get_cfg():

    lpc = LinchpinCliContext()
    lpc.load_config()
    cfg_value = lpc.get_cfg(section='lp', key='pkg')

    assert_equal(cfg_value, config_data['lp']['pkg'])


@with_setup(setup_context_data)
def test_set_cfg():

    lpc = LinchpinCliContext()
    lpc.load_config(config_path)
    lpc.set_cfg('test', 'key', 'value')

    assert_equal(lpc.get_cfg('test', 'key'), lpc.cfgs['test']['key'])


@with_setup(setup_context_data)
def test_load_global_evars():

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.load_global_evars()

    # remove workspace key as it's not from the config file
    for k in lpc.evars.keys():
        if k == 'workspace':
            if lpc.evars.get(k):
                lpc.evars.pop(k)
            if evars_data.get(k):
                evars_data.pop(k)

    assert_dict_equal(evars_data, lpc.evars)


@with_setup(setup_context_data)
def test_get_evar():

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.load_global_evars()
    evar_value = lpc.get_evar('_async')

    assert_equal(evar_value, evars_data['_async'])


@with_setup(setup_context_data)
def test_set_evar():

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.load_global_evars()
    lpc.set_evar('test', 'me')

    assert_equal('me', lpc.evars['test'])


@with_setup(setup_context_data)
def test_logging_setup():

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.set_cfg('logger',
                'file',
                config_data['logger']['file'])
    lpc.setup_logging()

    assert os.path.isfile(logfile)


@with_setup(setup_context_data)
def test_log_msg():

    # This test assumes the default message format found around line 139
    # of linchpin/cli/context.py

    lvl=logging.DEBUG
    msg = 'Test Msg'
    regex = '^{0}.*{1}'.format(logging.getLevelName(lvl), msg)

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.set_cfg('logger',
                'file',
                config_data['logger']['file'])
    lpc.setup_logging()
    lpc.log(msg, level=lvl)

    with open(logfile) as f:
        line = f.readline()

    assert_regexp_matches(line, regex)

@with_setup(setup_context_data)
def test_log_state():

    lvl=logging.DEBUG
    msg = '{0}: State Msg'.format(logging.getLevelName(lvl))
    regex = '^{0}.*STATE - {1}'.format(logging.getLevelName(lvl), msg)

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.set_cfg('logger',
                'file',
                config_data['logger']['file'])
    lpc.setup_logging()
    lpc.log_state(msg)

    with open(logfile) as f:
        line = f.readline()

    assert_regexp_matches(line, regex)

@with_setup(setup_context_data)
def test_log_info():

    lvl=logging.INFO
    msg = 'Info Msg'
    regex = '^{0}.*{1}'.format(logging.getLevelName(lvl), msg)

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.set_cfg('logger',
                'file',
                config_data['logger']['file'])
    lpc.setup_logging()
    lpc.log_info(msg)

    with open(logfile) as f:
        line = f.readline()

    assert_regexp_matches(line, regex)

@with_setup(setup_context_data)
def test_log_debug():

    lvl=logging.DEBUG
    msg = 'Debug Msg'
    regex = '^{0}.*{1}'.format(logging.getLevelName(lvl), msg)

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.set_cfg('logger',
                'file',
                config_data['logger']['file'])
    lpc.setup_logging()
    lpc.log_debug(msg)

    with open(logfile) as f:
        line = f.readline()

    assert_regexp_matches(line, regex)


def main():

    tlc = TestLinchpinCliContext()
    tlc.setup_load_config()
    tlc.test_load_config()

if __name__ == '__main__':
    sys.exit(main())



from click import Option, UsageError


class MutuallyExclusiveOption(Option):
    def __init__(self, *args, **kwargs):
        self.mutually_exclusive = set(kwargs.pop('mutually_exclusive', []))
        help = kwargs.get('help', '')
        if self.mutually_exclusive:
            ex_str = ', '.join(self.mutually_exclusive)
            kwargs['help'] = help + (
                ' NOTE: This argument is mutually exclusive with '
                ' arguments: [' + ex_str + '].'
            )
        super(MutuallyExclusiveOption, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        if self.mutually_exclusive.intersection(opts) and self.name in opts:
            raise UsageError(
                "Illegal usage: `{0}` is mutually exclusive with "
                "arguments `{1}`.".format(
                    self.name,
                    ', '.join(self.mutually_exclusive)
                )
            )

        return super(MutuallyExclusiveOption, self).handle_parse_result(
            ctx,
            opts,
            args
        )



from tinydb import TinyDB
from tinydb.storages import JSONStorage
from tinydb.operations import add
from tinydb.operations import set as tinySet
from tinydb.middlewares import CachingMiddleware
from collections import defaultdict
from .basedb import BaseDB


def usedb(func):
    def func_wrapper(*args, **kwargs):
        args[0]._opendb()
        x = func(*args, **kwargs)
        args[0]._closedb()
        return x
    return func_wrapper


class TinyRunDB(BaseDB):

    def __init__(self, conn_str):
        self.name = 'TinyRunDB'
        self.conn_str = conn_str
        self.default_table = 'linchpin'


    def _opendb(self):
        self.middleware = CachingMiddleware(JSONStorage)
        self.middleware.WRITE_CACHE_SIZE = 500
        self.db = TinyDB(self.conn_str, storage=self.middleware,
                         default_table=self.default_table)


    def __str__(self):
        if self.conn_str:
            return "{0} at {2}".format(self.name, self.conn_str)
        return "{0} at {1}".format(self.name, 'None')


    @property
    def schema(self):
        return self._schema


    @schema.setter
    def schema(self, schema):
        self._schema = dict()
        self._schema.update(schema)


    @usedb
    def init_table(self, table):
        t = self.db.table(name=table)
        return t.insert(self.schema)


    @usedb
    def update_record(self, table, run_id, key, value):
        t = self.db.table(name=table)
        tx_rec = t.get(eid=run_id).get("outputs", [])
        if len(tx_rec) > 0 and isinstance(value, list):
            # fetch the resources dict, index
            # by filtering them from outputs list
            res_list = [(idx, x) for idx, x in enumerate(tx_rec)
                        if "resources" in x]
            if len(res_list) != 0:
                res_idx = res_list[0][0]
                resources = res_list[0][1]
                if "resources" in value[0]:
                    de = defaultdict(list, resources["resources"])
                    for i, j in value[0]["resources"].items():
                        de[i].extend(j)
                    de = {"resources": de}
                    tx_rec[res_idx] = de
                    return t.update(tinySet(key, [de]), eids=[run_id])
        return t.update(add(key, value), eids=[run_id])


    @usedb
    def get_tx_record(self, tx_id):

        t = self.db.table(name='linchpin')
        return t.get(eid=tx_id)


    @usedb
    def get_tx_records(self, tx_ids):

        txs = {}
        t = self.db.table(name='linchpin')
        for tx_id in tx_ids:
            txs[tx_id] = t.get(eid=tx_id)

        return txs


    @usedb
    def get_record(self, table, action='up', run_id=None):

        t = self.db.table(name=table)
        if not run_id:
            run_id = len(t.all())
            if not run_id:
                return (None, 0)

            for rid in range(int(run_id), 0, -1):
                record = t.get(eid=int(rid))
                if record and record['action'] == action:
                    return (record, int(rid))
        else:
            record = t.get(eid=int(run_id))
            if record:
                return(record, int(run_id))


        return (None, 0)


    @usedb
    def get_records(self, table, count=10):
        records = {}
        if table in self.db.tables():
            t = self.db.table(name=table)
            if len(t.all()):
                start = len(t)
                if count == 'all':
                    end = 0
                else:
                    end = start - count
                for i in range(start, end, -1):
                    records[i] = t.get(doc_id=i)
        return records


    @usedb
    def get_tables(self):

        tables = self.db.tables()
        tables.remove(self.default_table)

        return tables


    def remove_record(self, table, key, value):
        pass


    def search(self, table, key=None):
        t = self.db.table(name=table)
        if key:
            return t.search(key)
        return t.all()


    def query(self, table, query):
        pass


    def purge(self, table=None):
        if table:
            return self.db.purge_table(table)
        return self.db.purge_tables()


    def _closedb(self):
        self.db.close()



import os
import sys
import ansible
from collections import namedtuple
from contextlib import contextmanager

ansible24 = float(ansible.__version__[0:3]) >= 2.4
ansible_version = float(ansible.__version__[0:3])


# CentOS 6 EPEL provides an alternate Jinja2 package
# used by the imports below - Ansible uses Jinja2 here

try:
    from .callbacks import PlaybookCallback
    from ansible.parsing.dataloader import DataLoader
    from ansible.executor.playbook_executor import PlaybookExecutor
except ImportError:
    sys.path.insert(0, '/usr/lib/python2.6/site-packages/Jinja2-2.6-py2.6.egg')
    from .callbacks import PlaybookCallback
    from ansible.parsing.dataloader import DataLoader
    from ansible.executor.playbook_executor import PlaybookExecutor

if ansible24:
    from ansible.vars.manager import VariableManager
    from ansible.inventory.manager import InventoryManager as Inventory
else:
    from ansible.inventory import Inventory
    from ansible.vars import VariableManager


@contextmanager
def suppress_stdout():
    """
    This context manager provides tooling to make Ansible's Display class
    not output anything when used
    """
    with open(os.devnull, "w") as devnull:
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = devnull
        sys.stderr = devnull
        try:
            yield
        finally:
            sys.stdout = old_stdout
            sys.stderr = old_stderr


def ansible_runner_2x(playbook_path,
                      extra_vars,
                      options=None,
                      inventory_src='localhost',
                      console=True):

    variable_manager = VariableManager()
    loader = DataLoader()
    variable_manager.extra_vars = extra_vars
    inventory = Inventory(loader=loader,
                          variable_manager=variable_manager,
                          host_list=inventory_src)
    passwords = {}
    pbex = PlaybookExecutor([playbook_path],
                            inventory,
                            variable_manager,
                            loader,
                            options,
                            passwords)

    return pbex



def ansible_runner_24x(playbook_path,
                       extra_vars,
                       options=None,
                       inventory_src='localhost',
                       console=True):

    loader = DataLoader()
    variable_manager = VariableManager(loader=loader)
    variable_manager.extra_vars = extra_vars
    inventory = Inventory(loader=loader, sources=[inventory_src])
    variable_manager.set_inventory(inventory)
    passwords = {}

    pbex = PlaybookExecutor([playbook_path],
                            inventory,
                            variable_manager,
                            loader,
                            options,
                            passwords)
    return pbex


def ansible_runner(playbook_path,
                   module_path,
                   extra_vars,
                   inventory_src='localhost',
                   verbosity=1,
                   console=True):
    """
    Uses the Ansible API code to invoke the specified linchpin playbook
    :param playbook: Which ansible playbook to run (default: 'up')
    :param console: Whether to display the ansible console (default: True)
    """

    # note: It may be advantageous to put the options into the context and pass
    # that onto this method down the road. The verbosity flag would just live
    # in options and we could set the defaults.

    # module path cannot accept list in ansible 2.3.x versions
    if ansible_version <= 2.3:
        module_path = ":".join(module_path)

    connect_type = 'ssh'
    if 'localhost' in inventory_src:
        extra_vars["ansible_python_interpreter"] = sys.executable
        connect_type = 'local'

    Options = namedtuple('Options', ['connection',
                                     'module_path',
                                     'forks',
                                     'become',
                                     'become_method',
                                     'become_user',
                                     'listhosts',
                                     'listtasks',
                                     'listtags',
                                     'syntax',
                                     'remote_user',
                                     'private_key_file',
                                     'ssh_common_args',
                                     'ssh_extra_args',
                                     'sftp_extra_args',
                                     'scp_extra_args',
                                     'verbosity',
                                     'check',
                                     'diff',
                                     ])

    options = Options(connection=connect_type,
                      module_path=module_path,
                      forks=100,
                      become=False,
                      become_method='sudo',
                      become_user='root',
                      listhosts=False,
                      listtasks=False,
                      listtags=False,
                      syntax=False,
                      remote_user=None,
                      private_key_file=None,
                      ssh_common_args=None,
                      ssh_extra_args=None,
                      sftp_extra_args=None,
                      scp_extra_args=None,
                      verbosity=verbosity,
                      check=False,
                      diff=False
                      )

    if ansible24:
        pbex = ansible_runner_24x(playbook_path,
                                  extra_vars,
                                  options,
                                  inventory_src=inventory_src,
                                  console=console)
    else:
        pbex = ansible_runner_2x(playbook_path,
                                 extra_vars,
                                 options,
                                 inventory_src=inventory_src,
                                 console=console)
    if ansible_version >= 2.5:
        cb = PlaybookCallback(options=options, ansible_version=ansible_version)
    else:
        cb = PlaybookCallback(options=options)

    if not console:
        results = {}
        return_code = 0
        with suppress_stdout():
            pbex._tqm._stdout_callback = cb
        return_code = pbex.run()
        results = cb.results
        return return_code, results
    else:
        # the console only returns a return_code
        pbex._tqm._stdout_callback = None
        return_code = pbex.run()
        return return_code, None



#!/usr/bin/env python


def render_os_server_insts(res_def, os_resource_name):

    output = []
    if "count" in res_def:
        server_names = [os_resource_name + str(i)
                        for i in range(1, res_def["count"] + 1)]
        for sname in server_names:
            server_dict = {}
            server_dict['name'] = sname
            if 'additional_volumes' in res_def:
                server_dict['volumes'] = []
                for vol in res_def.get("additional_volumes", []):
                    a_vol = {}
                    a_vol.update(vol)
                    a_vol["name"] = vol["name"] + "-" + server_dict["name"]
                    a_vol["server_name"] = server_dict["name"]
                    server_dict["volumes"].append(a_vol)
            output.append(server_dict)
    else:
        return res_def
    return output


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'render_os_server_insts': render_os_server_insts
        }



#!/usr/bin/env python


def format_libvirt_networks(networks,):
    output = ""
    for network in networks:
        output += " --network " + network.get('name', 'default')
    return output


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'format_libvirt_networks': format_libvirt_networks
        }



#!/usr/bin/env python

import os
import ast
import sys
import json
import time
import errno
import hashlib

from uuid import getnode as get_mac
from collections import OrderedDict

from linchpin.ansible_runner import ansible_runner

from linchpin.hooks.state import State
from linchpin.hooks import LinchpinHooks

from linchpin.rundb.basedb import BaseDB
from linchpin.rundb.drivers import DB_DRIVERS

from linchpin.exceptions import ActionError
from linchpin.exceptions import LinchpinError
from linchpin.exceptions import SchemaError
from linchpin.exceptions import TopologyError
from linchpin.exceptions import ValidationError
from linchpin.exceptions import ValidationErrorHandler

from linchpin.utils.validator import AnyofValidator

from linchpin.InventoryFilters import GenericInventory


class LinchpinAPI(object):

    def __init__(self, ctx):
        """
        LinchpinAPI constructor

        :param ctx: context object from context.py

        """

        self.__meta__ = "API"

        self.ctx = ctx

        self.hook_state = None
        self._hook_observers = []
        self.playbook_pre_states = self.get_cfg('playbook_pre_states',
                                                {'up': 'preup',
                                                 'destroy': 'predestroy'})
        self.playbook_post_states = self.get_cfg('playbook_post_states',
                                                 {'up': 'postup',
                                                  'destroy': 'postdestroy',
                                                  'postinv': 'inventory'})
        self.hooks = LinchpinHooks(self)

        self.target_data = {}

        base_path = '/'.join(os.path.dirname(__file__).split('/')[0:-1])
        pkg = self.get_cfg(section='lp', key='pkg', default='linchpin')
        lp_path = '{0}/{1}'.format(base_path, pkg)
        self.pb_ext = self.get_cfg('extensions', 'playbooks', default='.yml')

        # get external_provider_path
        xp_path = self.get_cfg('lp',
                               'external_providers_path',
                               default='').split(':')

        pb_path = '{0}/{1}'.format(lp_path,
                                   self.get_evar('playbooks_folder',
                                                 default='provision'))
        self.pb_path = [pb_path]

        for path in xp_path:
            self.pb_path.append(os.path.expanduser(path))

        self.set_evar('lp_path', lp_path)
        self.set_evar('pb_path', self.pb_path)
        self.set_evar('from_api', True)
        self.workspace = self.get_evar('workspace')


    def setup_rundb(self):
        """
        Configures the run database parameters, sets them into extra_vars
        """

        rundb_conn = self.get_cfg(section='lp',
                                  key='rundb_conn')
        rundb_type = self.get_cfg(section='lp',
                                  key='rundb_type',
                                  default='TinyRunDB')
        rundb_conn_type = self.get_cfg(section='lp',
                                       key='rundb_conn_type',
                                       default='file')
        self.rundb_hash = self.get_cfg(section='lp',
                                       key='rundb_hash',
                                       default='sha256')

        if rundb_conn_type == 'file':

            rundb_conn_f = rundb_conn.replace('::mac::', str(get_mac()))
            rundb_conn_f = rundb_conn_f.replace('{{ workspace }}',
                                                self.workspace)
            rundb_conn_f = os.path.realpath(os.path.expanduser(rundb_conn_f))
            rundb_conn_dir = os.path.dirname(rundb_conn_f)

            if not os.path.exists(rundb_conn_dir):
                try:
                    os.makedirs(rundb_conn_dir)
                except OSError as exc:
                    if (exc.errno == errno.EEXIST and
                            os.path.isdir(rundb_conn_dir)):
                        pass
                    else:
                        raise

        self.set_evar('rundb_type', rundb_type)
        self.set_evar('rundb_conn', rundb_conn_f)
        self.set_evar('rundb_hash', self.rundb_hash)

        return BaseDB(DB_DRIVERS[rundb_type], rundb_conn_f)


    def get_cfg(self, section=None, key=None, default=None):
        """
        Get cfgs value(s) by section and/or key, or the whole cfgs object

        :param section: section from ini-style config file

        :param key: key to get from config file, within section

        :param default: default value to return if nothing is found.
        """

        return self.ctx.get_cfg(section=section, key=key, default=default)


    def set_cfg(self, section, key, value):
        """
        Set a value in cfgs. Does not persist into a file,
        only during the current execution.

        :param section: section within ini-style config file

        :param key: key to use

        :param value: value to set into section within config file
        """

        self.ctx.set_cfg(section, key, value)


    def get_evar(self, key=None, default=None):
        """
        Get the current evars (extra_vars)

        :param key: key to use

        :param default: default value to return if nothing is found
        (default: None)
        """

        return self.ctx.get_evar(key=key, default=default)


    def set_evar(self, key, value):
        """
        Set a value into evars (extra_vars). Does not persist into a file,
        only during the current execution.

        :param key: key to use

        :param value: value to set into evars
        """

        self.ctx.set_evar(key, value)


    @property
    def hook_state(self):
        """
        getter function for hook_state property of the API object
        """

        return self.hook_state


    @hook_state.setter
    def hook_state(self, hook_state):
        """
        hook_state property setter , splits the hook_state string in
        subhook_state and sets linchpin.hook_state object

        :param hook_state: valid hook_state string mentioned in linchpin.conf
        """

        # call run_hooks after hook_state is being set
        if hook_state is None:
            return
        else:
            self.ctx.log_debug('hook {0} initiated'.format(hook_state))
            self._hook_state = State(hook_state, None, self.ctx)

            for callback in self._hook_observers:
                callback(self._hook_state)


    def bind_to_hook_state(self, callback):
        """
        Function used by LinchpinHooksclass to add callbacks

        :param callback: callback function
        """

        self._hook_observers.append(callback)


    def lp_journal(self, view='target', targets=[],
                   fields=None, count=1, tx_ids=None):

        rundb = self.setup_rundb()

        journal = {}

        if not len(targets):
            targets = rundb.get_tables()

        if view == 'target':
            # initialize rundb table
            dateformat = self.get_cfg('logger',
                                      'dateformat',
                                      default='%m/%d/%Y %I:%M:%S %p')
            for target in targets:
                tgt_records = rundb.get_records(table=target, count=count)

                if fields and 'start' in fields:
                    for run_id, record in tgt_records.iteritems():
                        st = record.get('start')
                        strt = float(st) if len(st) else float(1000000000.0)
                        start = time.localtime(strt)
                        record['start'] = time.strftime(dateformat, start)

                if fields and 'end' in fields:
                    for run_id, record in tgt_records.iteritems():
                        en = record.get('end')
                        endt = float(st) if len(en) else float(1000000000.0)
                        end = time.localtime(endt)
                        record['end'] = time.strftime(dateformat, end)


                journal[target] = tgt_records
        if view == 'tx':
            if len(tx_ids):
                journal = rundb.get_tx_records(tx_ids)
            else:
                journal = rundb.get_records('linchpin', count=count)

        return journal


    def _find_playbook_path(self, playbook):

        for path in self.pb_path:
            p = '{0}/{1}{2}'.format(path, playbook, self.pb_ext)

            if os.path.exists(os.path.expanduser(p)):
                return path

        raise LinchpinError("playbook '{0}' not found in"
                            " path: {1}".format(playbook, self.pb_path))


    def _fix_broken_topologies(self, res_grp, res_grp_type):
        """
        Because the new tooling requires resource_definitions, both beaker
        and openshift didn't previously suport this section in topologies.
        This function is called if resource_definitions are not included
        in the topology. The topology will be given a resource_definitions
        section.

        :param res_grp: The resource group to update

        :param res_grp_type: Which type to convert (this may not matter)
        """

        res_defs = {}
        # with beaker, there will only be one
        # resource_definition upon conversion
        if res_grp_type == 'beaker':
            res_defs['role'] = 'bkr_server'
            res_defs['whiteboard'] = res_grp.pop('whiteboard',
                                                 'Provisioned with LinchPin')
            res_defs['job_group'] = res_grp.pop('job_group')
            res_defs['recipesets'] = res_grp.pop('recipesets')
            res_defs['cancel_message'] = res_grp.pop('cancel_message',
                                                     'Canceled by LinchPin')
            res_defs['max_attempts'] = res_grp.pop('max_attempts', 60)
            res_defs['attempt_wait_time'] = res_grp.pop('attempt_wait_time',
                                                        60)
            res_grp['resource_definitions'] = [res_defs]

            return res_grp

        if res_grp_type == 'openshift':
            res_defs = res_grp.pop('resources')

            count = 0
            for res in res_defs:
                res['name'] = 'openshift-res_{0}'.format(str(count))
                count += 1
                if res.get('inline_data'):
                    res['role'] = 'openshift_inline'
                    res['data'] = res.pop('inline_data')
                if res.get('file_reference'):
                    res['role'] = 'openshift_external'
                    res['filiename'] = res.pop('file_reference')

            res_grp['resource_definitions'] = res_defs

            # the old openshift role put credentials in the
            # resource group definition. Moving to credentials
            creds = {}
            creds['api_endpoint'] = res_grp.pop('api_endpoint')
            creds['api_token'] = res_grp.pop('api_token')
            res_grp['credentials'] = creds

            return res_grp


    def _convert_topology(self, topology):
        """
        For backward compatiblity, convert the old topology format
        into the new format. Should be pretty straightforward and simple.

        ;param topology: topology dictionary
        """
        try:
            res_grps = topology.get('resource_groups')
            if res_grps:
                for res_grp in res_grps:
                    if 'res_group_type' in res_grp.keys():
                        res_grp['resource_group_type'] = (
                            res_grp.pop('res_group_type'))

                    if 'res_defs' in res_grp.keys():
                        res_grp['resource_definitions'] = (
                            res_grp.pop('res_defs'))

                    res_defs = res_grp.get('resource_definitions')
                    if not res_defs:
                        # this means it's either a beaker or openshift topology
                        res_grp_type = res_grp.get('resource_group_type')

                        res_group = self._fix_broken_topologies(res_grp,
                                                                res_grp_type)
                        res_defs = res_group.get('resource_definitions')
                        res_grp['resource_definitions'] = res_defs

                    if res_defs:
                        for res_def in res_defs:
                            if 'res_name' in res_def.keys():
                                res_def['name'] = res_def.pop('res_name')
                            if 'type' in res_def.keys():
                                res_def['role'] = res_def.pop('type')
                            if 'res_type' in res_def.keys():
                                res_def['role'] = res_def.pop('res_type')
                            if 'count' in res_def.keys():
                                res_def['count'] = int(res_def.pop('count'))
                    else:
                        raise TopologyError("'resource_definitions' do not"
                                            " validate in topology"
                                            " ({0})".format(topology))
            else:
                raise TopologyError("'resource_groups' do not validate"
                                    " in topology ({0})".format(topology))

        except Exception:
            raise LinchpinError("Unknown error converting schema. Check"
                                " template data")

    def _convert_layout(self, layout_data):
        """
        Convert the layout to retain order of the layout hosts
        ;param layout_data: layout_data
        """
        layout_json = layout_data
        layout_hosts = []
        ihosts = layout_data["inventory_layout"]["hosts"]
        for k in ihosts:
            layout_host = {}
            layout_host["name"] = k
            for key in ihosts[k]:
                layout_host[key] = ihosts[k][key]
            layout_hosts.append(layout_host)
        layout_json["inventory_layout"]["hosts"] = layout_hosts
        return layout_json


    def validate_topology(self, topology):
        """
        Validate the provided topology against the schema

        ;param topology: topology dictionary
        """

        res_grps = topology.get('resource_groups')
        resources = []

        for group in res_grps:
            res_grp_type = (group.get('resource_group_type') or
                            group.get('res_group_type'))

            pb_path = self._find_playbook_path(res_grp_type)

            try:
                sp = "{0}/roles/{1}/files/schema.json".format(pb_path,
                                                              res_grp_type)
                schema = json.load(open(sp))
            except Exception as e:
                raise LinchpinError("Error with schema: '{0}'"
                                    " {1}".format(sp, e))

            res_defs = group.get('resource_definitions')


            # preload this so it will validate against the schema
            document = {'res_defs': res_defs}
            v = AnyofValidator(schema,
                               error_handler=ValidationErrorHandler)

            if not v.validate(document):
                try:
                    err = self._gen_error_msg("", "", v.errors)
                    raise SchemaError(err)
                except NotImplementedError as e:
                    # we shouldn't have this issue using cerberus >= 1.2, but
                    # this is here just in case an older version has to be used
                    print("There was an error validating your schema, but we\
                          can't seem to format it for you")
                    print("Here's the raw error data in case you want to go\
                          through it by hand:")
                    print(v._errors)

            resources.append(group)

        return resources


    def _gen_error_msg(self, prefix, section, error):
        # set the prefix for this subtree
        if section != "":
            if prefix != "":
                prefix += "[" + str(section) + "]"
            else:
                prefix = str(section)

        if isinstance(error, str):
            if prefix == "":
                return error
            return prefix + ": " + error + os.linesep
        elif isinstance(error, list):
            msg = ""
            for i, e in enumerate(error):
                # we don't need to change the Vprefix here
                msg += self._gen_error_msg(prefix, "", e)
            return msg
        else:  # in this case, error is a dict
            msg = ""
            for key, val in error.iteritems():
                msg += self._gen_error_msg(prefix, key, val)
            return msg


    def validate_layout(self, layout):
        """
        Validate the provided layout against the schema

        :param layout: layout dictionary
        """

        pb_path = self._find_playbook_path("layout")
        try:
            sp = "{0}/roles/common/files/schema.json".format(pb_path)

            schema = json.load(open(sp))
        except Exception as e:
            raise LinchpinError("Error with schema: '{0}' {1}".format(sp, e))

        v = AnyofValidator(schema)

        if not v.validate(layout):
            raise SchemaError('Schema validation failed: {0}'.format(v.errors))


    def generate_inventory(self, resource_data, layout, inv_format="cfg",
                           topology_data={}, config_data={}):
        inv = GenericInventory.GenericInventory(inv_format=inv_format)
        inventory = inv.get_inventory(resource_data, layout, topology_data,
                                      config_data)
        return inventory

    def get_pf_data_from_rundb(self, targets, run_id=None, tx_id=None):
        """
        This function takes the action and provision_data, returns the
        pinfile data

        :param targets: A list of targets for which to get the data

        :param targets: Tuple of target(s) for which to gather data.

        :param run_id: run_id associated with target (Default: None)

        :param tx_id: tx_id for which to gather data (Default: None)
        """

        rundb = self.setup_rundb()

        if run_id and tx_id:
            raise ActionError("'run_id' and 'tx_id' are mutually exclusive")

        pf_data = {}
        pinfile = OrderedDict()

        if run_id:
            for target in targets:
                pf_data[target] = rundb.get_record(target,
                                                   action='up',
                                                   run_id=run_id)
        if tx_id:
            record = rundb.get_tx_record(tx_id)

            if not record or not len(record):
                return None

            if len(targets):
                for tgts in record['targets']:
                        for tgt, data in tgts.iteritems():
                            run_id = int(data.keys()[0])
                            if tgt in targets:
                                tgt_data = (rundb.get_record(tgt,
                                            action=record['action'],
                                            run_id=run_id))
                                pf_data[tgt] = tgt_data
            else:
                for tgts in record['targets']:
                    for tgt, data in tgts.iteritems():
                        run_id = int(data.keys()[0])
                        tgt_data = (rundb.get_record(tgt,
                                    action=record['action'],
                                    run_id=run_id))
                        pf_data[tgt] = tgt_data


        for t, data in pf_data.iteritems():
            topo_data = data[0]['inputs'][0].get('topology_data')
            layout_data = data[0]['inputs'][0].get('layout_data')
            hooks_data = data[0]['inputs'][0].get('hooks_data')

            pinfile[t] = {}
            pinfile[t]['topology'] = topo_data
            pinfile[t]['run_id'] = data[1]
            if layout_data:
                pinfile[t]['layout'] = layout_data
            if hooks_data:
                pinfile[t]['hooks'] = hooks_data

        return pinfile


    def do_action(self, provision_data, action='up', run_id=None, tx_id=None):
        """
        This function takes provision_data, and executes the given
        action for each target within the provision_data disctionary.

        :param provision_data: PinFile data as a dictionary, with
        target information

        :param action: Action taken (up, destroy, etc). (Default: up)

        :param run_id: Provided run_id to duplicate/destroy (Default: None)

        :param tx_id: Provided tx_id to duplicate/destroy (Default: None)

        .. .note:: The `run_id` value differs from the `rundb_id`, in that
                   the `run_id` is an existing value in the database.
                   The `rundb_id` value is created to store the new record.
                   If the `run_id` is passed, it is used to collect an existing
                   `uhash` value from the given `run_id`, which is in turn used
                   to perform an idempotent reprovision, or destroy provisioned
                   resources.
        """
        ansible_console = False
        if self.ctx.cfgs.get('ansible'):
            ansible_console = (
                ast.literal_eval(self.get_cfg('ansible',
                                              'console',
                                              default='False')))

        if not ansible_console:
            ansible_console = bool(self.ctx.verbosity)

        results = {}


        return_code = 99
        vault_pass = self.get_evar('vault_pass')
        if vault_pass == '':
            self.set_evar('vault_pass',
                          self.ctx.get_cfg('evars',
                                           'vault_pass',
                                           default=''))

        for target in provision_data.keys():
            if not isinstance(provision_data[target], dict):
                raise LinchpinError("Target '{0}' does not"
                                    " exist.".format(target))

        targets = [x.lower() for x in provision_data.keys()]
        if 'linchpin' in targets:
            raise LinchpinError("Target 'linchpin' is not allowed.")

        for target in provision_data.keys():
            if target == 'cfgs':
                continue

            self.ctx.log_debug("Processing target: {0}".format(target))

            results[target] = {}
            self.set_evar('target', target)

            rundb = self.setup_rundb()

            if tx_id:
                record = rundb.get_tx_record(tx_id)
                run_id = (record['targets'][0][target].keys()[0])


            rundb_schema = json.loads(self.get_cfg(section='lp',
                                      key='rundb_schema'))
            rundb.schema = rundb_schema
            self.set_evar('rundb_schema', rundb_schema)

            start = time.time()
            st_uhash = int(start * 1000)
            uhash = None

            # generate a new rundb_id
            # (don't confuse it with an already existing run_id)
            rundb_id = rundb.init_table(target)
            orig_run_id = rundb_id

            uhash_length = self.get_cfg('lp', 'rundb_uhash_length')
            uhash_len = int(uhash_length)
            if not run_id:
                uh = hashlib.new(self.rundb_hash,
                                 ':'.join([target, str(tx_id),
                                          str(rundb_id), str(st_uhash)]))
                uhash = uh.hexdigest()[:uhash_len]

            if action == 'destroy' or run_id:
                # look for the action='up' records to destroy
                data, orig_run_id = rundb.get_record(target,
                                                     action='up',
                                                     run_id=run_id)

                if data:
                    uhash = data.get('uhash')
                    self.ctx.log_debug("using data from"
                                       " run_id: {0}".format(run_id))

            elif action not in ['up', 'destroy']:
                # it doesn't appear this code will will execute,
                # but if it does...
                raise LinchpinError("Attempting '{0}' action on"
                                    " target: '{1}' failed. Not an"
                                    " action.".format(action, target))


            self.ctx.log_debug('rundb_id: {0}'.format(rundb_id))
            self.ctx.log_debug('uhash: {0}'.format(uhash))

            rundb.update_record(target, rundb_id, 'uhash', uhash)
            rundb.update_record(target, rundb_id, 'start', str(start))
            rundb.update_record(target, rundb_id, 'action', action)

            self.set_evar('orig_run_id', orig_run_id)
            self.set_evar('rundb_id', rundb_id)
            self.set_evar('uhash', uhash)

            topology_data = provision_data[target].get('topology')

            # if validation fails the first time, convert topo from old -> new
            try:
                resources = self.validate_topology(topology_data)
            except (SchemaError, KeyError):
                # if topology fails, try converting from old to new style
                try:
                    self._convert_topology(topology_data)
                    resources = self.validate_topology(topology_data)
                except SchemaError:
                    raise ValidationError("Topology '{0}' does not validate.  "
                                          "For more information run `linchpin "
                                          "validate`".format(topology_data))


            self.set_evar('topo_data', topology_data)

            rundb.update_record(target,
                                rundb_id,
                                'inputs',
                                [
                                    {'topology_data':
                                     provision_data[target]['topology']}
                                ])

            if provision_data[target].get('layout', None):
                l_data = provision_data[target]['layout']
                try:
                    self.validate_layout(l_data)
                except SchemaError:
                    raise ValidationError("Layout '{0}' does not"
                                          " validate".format(l_data))

                provision_data[target]['layout'] = self._convert_layout(l_data)
                self.set_evar('layout_data', provision_data[target]['layout'])

                rundb.update_record(target,
                                    rundb_id,
                                    'inputs',
                                    [
                                        {'layout_data':
                                         provision_data[target]['layout']}
                                    ])

            if provision_data[target].get('hooks', None):
                hooks_data = provision_data[target].get('hooks')
                self.set_evar('hooks_data', hooks_data)
                rundb.update_record(target,
                                    rundb_id,
                                    'inputs',
                                    [
                                        {'hooks_data':
                                         provision_data[target]['hooks']}
                                    ])

            if provision_data.get('cfgs', None):
                vars_data = provision_data[target].get('cfgs')
                self.set_evar('cfgs_data', vars_data)
                rundb.update_record(target,
                                    rundb_id,
                                    'cfgs',
                                    [
                                        {'user':
                                         provision_data['cfgs']}
                                    ])

            # note : changing the state triggers the hooks
            self.hooks.rundb = (rundb, rundb_id)
            self.pb_hooks = self.get_cfg('hookstates', action)
            self.ctx.log_debug('calling: {0}{1}'.format('pre', action))

            if (('pre' in self.pb_hooks) and not self.get_cfg('hook_flags',
                                                              'no_hooks')):
                self.hook_state = '{0}{1}'.format('pre', action)

            # FIXME need to add rundb data for hooks results

            # invoke the appropriate action
            return_code, results[target]['task_results'] = (
                self._invoke_playbooks(resources, action=action,
                                       console=ansible_console)
            )

            # logs the return_code back to console and quits
            # quits if return_code > 0
            self.ctx.log("Playbook Return code {0}".format(str(return_code)))
            if return_code > 0:
                sys.exit(return_code)

            if not return_code:
                self.ctx.log_state("Action '{0}' on Target '{1}' is "
                                   "complete".format(action, target))

            # FIXME Check the result[target] value here, and fail if applicable.
            # It's possible that a flag might allow more targets to run, then
            # return an error code at the end.

            # add post provision hook for inventory generation
            if (('inv' in self.pb_hooks) and not self.get_cfg('hook_flags',
                                                              'no_hooks')):
                self.hook_state = 'postinv'

            if (('post' in self.pb_hooks) and
                    (self.__meta__ == "API") and
                    not self.get_cfg('hook_flags', 'no_hooks')):
                self.hook_state = '{0}{1}'.format('post', action)

            end = time.time()

            rundb.update_record(target, rundb_id, 'end', str(end))
            rundb.update_record(target, rundb_id, 'rc', return_code)

            run_data = rundb.get_record(target,
                                        action=action,
                                        run_id=rundb_id)

            results[target]['rundb_data'] = {rundb_id: run_data[0]}


        # generate the linchpin_id and structure
        lp_schema = ('{"action": "", "targets": []}')

        rundb = self.setup_rundb()
        rundb.schema = json.loads(lp_schema)
        lp_id = rundb.init_table('linchpin')

        summary = {}

        for target, data in results.iteritems():
            for k, v in data['rundb_data'].iteritems():
                summary[target] = {k: {'rc': v['rc'], 'uhash': v['uhash']}}

        rundb.update_record('linchpin', lp_id, 'action', action)
        rundb.update_record('linchpin', lp_id, 'targets', [summary])

        lp_data = {lp_id: {'action': action,
                           'summary_data': summary,
                           'results_data': results}}

        return (return_code, lp_data)


    def do_validation(self, provision_data, old_schema=False):
        """
        This function takes provision_data, and attempts to validate the
        topologies for that data

        :param provision_data: PinFile data as a dictionary, with
        target information
        """

        results = {}


        return_code = 0

        for target in provision_data.keys():
            if not isinstance(provision_data[target], dict):
                raise LinchpinError("Target '{0}' does not"
                                    " exist.".format(target))

        targets = [x.lower() for x in provision_data.keys()]
        if 'linchpin' in targets:
            raise LinchpinError("Target 'linchpin' is not allowed.")

        for target in provision_data.keys():
            self.ctx.log_debug("Processing target: {0}".format(target))

            results[target] = {}
            self.set_evar('target', target)

            # validate topology
            topology_data = provision_data[target].get('topology')

            try:
                self.validate_topology(topology_data)
            except (SchemaError, KeyError) as e:
                # try to validate against old schema
                try:
                    self._convert_topology(topology_data)
                    self.validate_topology(topology_data)
                except SchemaError as s:
                    error = "errors:\n".format(target, topology_data)
                    if old_schema:
                        # there's an inline way of doing this with join() but
                        # this looks cleaner
                        for line in iter(str(s).splitlines(True)):
                            error += "\t" + line
                    else:
                        if type(e) == KeyError:
                            error += "\tfield res_defs['type'] is no longer"\
                                     "Please use 'role' instead"
                        else:
                            for line in iter(str(e).splitlines(True)):
                                error += "\t" + line

                    results[target]['topology'] = error
                    return_code += 1
                else:
                    results[target]['topology'] = "topology valid under old "\
                                                  "schema"


            else:
                results[target]['topology'] = "valid"

            # validate layout
            if provision_data[target].get('layout', None):
                l_data = provision_data[target]['layout']

                try:
                    self.validate_layout(l_data)
                except SchemaError as e:
                    error = "errors:".format(target, l_data)
                    for line in iter(str(e).splitlines(True)):
                        error += "\t" + line

                    results[target]['layout'] = error + "\n"

                else:
                    results[target]['layout'] = "valid"


        return return_code, results


    def get_run_data(self, tx_id, fields, targets=()):
        """
        Returns the RunDB for data from a specified field given a tx_id.
        The fields consist of the major sections in the RunDB (target
        view only). Those fields are action, start, end, inputs, outputs,
        uhash, and rc.

        :param tx_id: tx_id to search
        :param fields: Tuple of fields to retrieve for each record requested.
        :param targets: Tuple of targets to search from within the tx_ids
        """

        rundb = self.setup_rundb()

        tgt_run_ids = {}
        target_data = {}

        record = rundb.get_tx_record(tx_id)

        if not record or not len(record):
            return None


        # get run_ids to query
        if len(targets):
            for tgts in record['targets']:
                    for tgt, data in tgts.iteritems():
                        if tgt in targets:
                            tgt_run_ids[tgt] = int(data.keys()[0])
        else:
            for tgts in record['targets']:
                for tgt, data in tgts.iteritems():
                    tgt_run_ids[tgt] = int(data.keys()[0])

        for target, run_id in tgt_run_ids.iteritems():
            record = rundb.get_record(target, run_id=run_id, action='up')
            field_data = {}
            single_value_fields = ('action',
                                   'start',
                                   'end',
                                   'rc',
                                   'uhash')

            for field in fields:
                f = record[0].get(field)
                if f:
                    if field in single_value_fields:
                        field_data[field] = f
                    else:
                        data_array = {}
                        for fld in f:
                            for k, v in fld.iteritems():
                                if field == 'outputs':
                                    if isinstance(v, dict):
                                        values = v
                                    else:
                                        values = []
                                        for value in v:
                                            values.append(value)
                                    data_array[k] = values
                                else:
                                    data_array[k] = v

                            field_data[field] = data_array

            target_data[target] = field_data

        return target_data

    def _get_run_data_by_txid(self, tx_id=None):
        rundb = self.setup_rundb()
        latest_run_data = {}
        run_data = {}
        if tx_id is None:
            latest_run_data = rundb.get_records('linchpin', count=1)
            run_data = self.get_run_data(latest_run_data.keys()[0],
                                         ('outputs',
                                          'inputs',
                                          'cfgs'))
        else:
            latest_run_data = rundb.get_records('linchpin',
                                                count='all')
            latest_run_data = {tx_id: latest_run_data.get(tx_id)}
            run_data = self.get_run_data(tx_id,
                                         ('outputs',
                                          'inputs',
                                          'cfgs'))
        for k in latest_run_data:
            v = latest_run_data[k]
            target_group = v.get("targets", [])
            # Note:
            # target_group always returns a dict inside a list due to
            # rundb implemenatation. This code works for one PinFile run
            # might be subjected to change in future releases
            if len(target_group) == 0:
                continue
            else:
                target_group = target_group[0]
            for key in target_group.keys():
                target_group[key].update(run_data.get(key, {}))
            latest_run_data[k]["targets"] = [target_group]
        return latest_run_data

    def _get_module_path(self):
        module_paths = []
        module_folder = self.get_cfg('lp',
                                     'module_folder',
                                     default='library')
        for path in reversed(self.pb_path):
            module_paths.append('{0}/{1}/'.format(path, module_folder))
        return module_paths

    def _find_n_run_pb(self, pb_name, inv_src, console=True):
        pb_path = self._find_playbook_path(pb_name)
        playbook_path = '{0}/{1}{2}'.format(pb_path, pb_name, self.pb_ext)
        extra_vars = self.get_evar()
        return_code, res = ansible_runner(playbook_path,
                                          self._get_module_path(),
                                          extra_vars,
                                          inventory_src=inv_src,
                                          verbosity=self.ctx.verbosity,
                                          console=console)
        return return_code, res

    def _invoke_playbooks(self, resources={}, action='up', console=True,
                          providers=[]):
        """
        Uses the Ansible API code to invoke the specified linchpin playbook

        :param resources: dict of resources to provision
        :param action: Which ansible action to run (default: 'up')
        :param console: Whether to display the ansible console (default: True)
        """

        return_code = 0
        results = []

        self.set_evar('_action', action)
        self.set_evar('state', 'present')

        if action == 'setup' or action == 'ask_sudo_setup':
            self.set_evar('setup_providers', providers)
            return_code, res = self._find_n_run_pb(action,
                                                   "localhost",
                                                   console=console)
            if res:
                results.append(res)
            if not len(results):
                results = None
            return (return_code, results)

        if action == 'destroy':
            self.set_evar('state', 'absent')

        inventory_src = '{0}/localhost'.format(self.workspace)

        for resource in resources:
            self.set_evar('resources', resource)
            playbook = resource.get('resource_group_type')
            return_code, res = self._find_n_run_pb(playbook,
                                                   inventory_src,
                                                   console=console)
            if res:
                results.append(res)

        if not len(results):
            results = None

        return (return_code, results)



#!/usr/bin/env python

# flake8: noqa

import os
import sys

def get_mac_fun(text):
    # "mac address='52:54:00:cb:8a:d7'/>"
    strn = text[0]
    return strn.split("=")[-1].strip("'").strip("'/>")

class FilterModule(object):
    ''' A filter to fix interface's name format '''
    def filters(self):
        return {
            'get_mac': get_mac_fun
        }



import os
import ast
import shutil

try:
    import configparser
except ImportError:
        import ConfigParser as configparser


from abc import ABCMeta, abstractmethod
from linchpin.exceptions import LinchpinError


class Fetch(object):
    __metaclass__ = ABCMeta

    def __init__(self, ctx, fetch_type, dest, root='', root_ws='', ref=None):
        """
        """

        self.ctx = ctx
        self.fetch_type = fetch_type
        self.root = root
        self.root_ws = root_ws
        self.ref = ref
        self.tempdirs = []
        self.dest = os.path.abspath(os.path.realpath(dest))
        self.config_path = os.path.abspath(os.path.join(os.path.expanduser('~'),
                                           '.cache/linchpin/fetch.conf'))
        self.cfgs = self.read_cfg()

    @abstractmethod
    def fetch_files(self):
        pass

    def copy_files(self):

        if self.fetch_type == 'workspace':
            src_dir = '{0}/{1}'.format(self.td, self.root)
            self.copy_dir(src_dir, self.dest)
        else:
            self.transfer_section(self.fetch_type)


    def transfer_section(self, section):
        dest_dir = os.path.join(self.dest, section)
        dir_exists = True
        if section not in os.listdir(self.dest):
            dir_exists = False
            os.makedirs(dest_dir)

        src_dir = os.path.join('{0}/{1}'.format(self.td, self.root), section)
        if not os.path.exists(src_dir):
            if not dir_exists:
                shutil.rmtree(dest_dir)
            raise LinchpinError('The {0} directory does not exist in '
                                '{1}'.format(self.fetch_type, self.src))
        self.copy_dir(src_dir, dest_dir)


    def copy_dir(self, src, dest):

        for root, dirs, files in os.walk(src):
            files = [f for f in files if not f[0] == '.']
            dirs[:] = [d for d in dirs if not d[0] == '.']
            if not os.path.isdir(root):
                os.makedirs(root)
            for f in files:
                rel_path = root.replace(src, '').lstrip(os.sep)
                dest_path = os.path.join(dest, rel_path)

                if not os.path.isdir(dest_path):
                    try:
                        os.makedirs(dest_path)
                    except OSError as e:
                        if e.errno == 17:
                            raise LinchpinError('File {0} already exists'
                                                ' in destination directory'
                                                .format(e.filename))

                s_file = os.path.join(root, f)
                d_file = os.path.join(dest_path, f)

                # fetch.always_update_workspace flag determines whether or
                # not to update. can be overwritten on the cli with --nocache.
                cache_ws = (ast.literal_eval(
                            self.ctx.get_cfg('fetch', 'cache_ws',
                                             default='True')))

                cp_files = False
                if not cache_ws:
                    cp_files = True
                else:
                    if not os.path.exists(d_file):
                        cp_files = True
                    else:
                        cache_days = int(self.ctx.get_cfg('fetch',
                                                          'cache_days',
                                                          default=1))
                        s_file_mtime = int(os.stat(s_file).st_mtime)
                        d_file_mtime = int(os.stat(d_file).st_mtime)

                        if (s_file_mtime - d_file_mtime) >= cache_days:
                            cp_files = True

                if cp_files:

                    try:
                        if (os.path.islink(s_file) and
                                os.path.exists(os.readlink(s_file))):
                            s_file = os.readlink(s_file)

                        shutil.copy2(s_file, d_file)
                    except (IOError, OSError) as e:
                        self.ctx.log_state(e)


    def read_cfg(self):
        config = configparser.SafeConfigParser()
        config.optionxform = str

        cfgs = {}
        if not os.path.exists(self.config_path):
            config.add_section('http')
            config.add_section('git')
            with open(self.config_path, 'w') as configfile:
                config.write(configfile)
        else:
            config.read(self.config_path)

        for section in config.sections():
            cfgs[section] = {}
            if config.items(section) is None:
                continue
            for k, v in config.items(section):
                cfgs[section][k] = v
        return cfgs


    def write_cfg(self, section, key, value):
        config = configparser.SafeConfigParser()
        config.optionxform = str
        config.read(self.config_path)
        config.set(section, key, value)

        with open(self.config_path, 'w') as configfile:
            config.write(configfile)



#!/usr/bin/env python
from xml.etree.ElementTree import fromstring


def get_libvirt_files(output):
    files = []
    results = output['results']
    for result in results:
        stdout = result['stdout']
        myxml = fromstring(stdout)
        devices = myxml.findall('devices')
        for device in devices:
            disks = device.findall('disk')
            for disk in disks:
                if disk.attrib["type"] == 'file':
                    if len(disk.findall('source')) > 0:
                        source = disk.findall('source')[0]
                        files.append(source.attrib['file'])
    return files


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'get_libvirt_files': get_libvirt_files
        }



#!/user/bin/env python

import os
import json

from nose.tools import *

from linchpin.InventoryFilters import JSONInventoryFormatter

def setup_json_inventory_formatter():
    global formatter
    global inv

    formatter = JSONInventoryFormatter.JSONInventoryFormatter()
    subdir = 'layouts'
    base_path = '{0}'.format(os.path.dirname(
        os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))
    mock_path = '{0}/{1}/{2}'.format(lib_path, 'mockdata', subdir)
    
    template = 'parsed-layout.json'
    template_file = open(mock_path+'/'+template)
    inv = json.load(template_file)


@with_setup(setup_json_inventory_formatter)
def test_add_sections():
    """ 
    should add a list of sections to the config, as well as "all"
    """
    section_list = ['foo', 'bar']

    formatter.add_sections(section_list)
    sections = formatter.config
    assert_equal(set(sections), set(['foo','bar','all']))


@with_setup(setup_json_inventory_formatter)
def test_set_children():
    """
    Should set the children of each inventory host group in the config
    """
    empty_inv = dict()
    formatter.set_children(empty_inv)

    formatter.set_children(inv)

    host_group='OSEv3'
    # if one if these does not exist, a NoOptionError will be thrown
    for child in inv['host_groups'][host_group]['children']:
        formatter.config.get('{0}:children'.format(host_group), child)


@with_setup(setup_json_inventory_formatter)
def test_set_vars():
    """
    Should add the vars in each host_group to the config
    """
    empty_inv = dict()
    formatter.set_vars(empty_inv)

    formatter.add_sections(inv['host_groups'].keys())
    formatter.set_vars(inv)

    host_group='OSEv3'
    vars = formatter.config[host_group]['vars']
    for key, val in inv['host_groups'][host_group]['vars'].iteritems():
        assert_equal(val, vars[key])


@with_setup(setup_json_inventory_formatter)
def test_add_ips_to_groups():
    """
    """
    inven_hosts = inv['hosts']
    formatter.add_sections(inv['host_groups'].keys())
    formatter.add_ips_to_groups(inven_hosts, inv)
 

@with_setup(setup_json_inventory_formatter)
def test_add_common_vars():
    """
    """
    host_groups = inv['host_groups'].keys()
    formatter.add_sections(host_groups)
    formatter.set_vars(inv)
    formatter.add_ips_to_groups(host_groups, inv)
    formatter.add_common_vars(host_groups, inv)


@with_setup(setup_json_inventory_formatter)
def test_generate_inventory():
    """
    """
    inv_json = formatter.generate_inventory()
    # if the json is not valid, this will throw a ValueError and the test
    # will fail
    data = json.loads(inv_json)#!/user/bin/env python


from cerberus import errors as cerberus_errors


class LinchpinError(Exception):

    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


class HookError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)


class StateError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)


class ActionManagerError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)


class SchemaError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)


class ValidationError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)


class TopologyError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)


class ActionError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)


class ValidationErrorHandler(cerberus_errors.BasicErrorHandler):
    messages = cerberus_errors.BasicErrorHandler.messages.copy()
    messages[cerberus_errors.REQUIRED_FIELD.code] = "field '{field}' is "\
        + "required"
    messages[cerberus_errors.UNKNOWN_FIELD.code] = "field '{field}' could not "\
        + "be recognized within the schema provided"
    messages[cerberus_errors.BAD_TYPE.code] = "value for field '{field}' must "\
        + "be of type '{constraint}'"
    messages[cerberus_errors.UNALLOWED_VALUE.code] = "unallowed value " \
        + "'{value}' for field '{field}'. Allowed values are: {constraint}"



#!/usr/bin/env python


def filter_list_by_attr(output, attr):
    new_output = []
    for ele in output:
        if attr in ele:
            new_output.append(ele)
    return new_output


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'filter_list_by_attr': filter_list_by_attr
        }



#!/usr/bin/env python

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from .InventoryFilter import InventoryFilter


class DummyInventory(InventoryFilter):
    def get_host_data(self, topo, cfgs={}):
        """
        Returns a dict of hostnames or IP addresses for use in an Ansible
        inventory file, based on available data. Only a single hostname or IP
        address will be returned per instance, so as to avoid duplicate runs of
        Ansible on the same host via the generated inventory file.

        Each hostname contains mappings of any variable that was defined in the
        cfgs section of the PinFile (e.g. __IP__) to the value in the field that
        corresponds with that variable in the cfgs.

        By default, the hostname will be the system field returned by dummy

        :param topo:
            linchpin Dummy resource data

        :param cfgs:
            map of config options from PinFile
        """

        host_data = {}
        var_data = cfgs.get('dummy', {})
        if var_data is None:
            var_data = {}
            var_data['__IP__'] = '__SELF__'
        for group in topo.get('dummy_res', []):
            for host in group['hosts']:
                host_data[host] = {}
                self.set_config_values(host_data[host], host, group, var_data)
        return host_data

    def get_host_ips(self, host_data):
        return host_data.keys()

    def get_inventory(self, topo, layout, config):
        if len(topo['dummy_res']) == 0:
            return ""
        host_data = self.get_host_data(topo, config)
        inven_hosts = self.get_host_ips(host_data)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO()
        self.config.write(output)
        return output.getvalue()

    def set_config_values(self, host_data, host, instance, cfgs={}):
        """
        """
        if cfgs is None:
            return
        if 'hostname' not in cfgs.keys():
            cfgs['hostname'] = '__IP__'
        for var in cfgs.keys():
            if var == 'hostname' and cfgs[var] == '__IP__':
                host_data[cfgs[var]] = host
            else:
                host_data[var] = self.config_value_helper(instance, cfgs[var])



#!/usr/bin/env python

import json

from .InventoryFormatter import InventoryFormatter


class JSONInventoryFormatter(InventoryFormatter):

    def __init__(self):
        InventoryFormatter.__init__(self)
        self.config = {}

    def add_sections(self, section_list):
        for section in section_list:
            self.config[section] = {}
        # adding a default section all
        if "all" not in self.config.keys():
            self.config["all"] = {}

    def set_children(self, inv):
        if 'host_groups' not in inv.keys():
            return
        for host_group in inv['host_groups']:
            if "children" in inv['host_groups'][host_group]:
                if not (host_group in self.config):
                    self.config[host_group] = {}
                self.config[host_group]["children"] = []
                for child in inv['host_groups'][host_group]['children']:
                    self.config[host_group]["children"].append(child)

    def set_vars(self, inv):
        if 'host_groups' not in inv.keys():
            return
        for host_group in inv['host_groups']:
            if "vars" in inv['host_groups'][host_group]:
                if not("vars" in self.config[host_group]):
                    self.config[host_group]["vars"] = {}
                host_grp_vars = inv['host_groups'][host_group]['vars']
                self.config[host_group]["vars"].update(host_grp_vars)
        for host in inv['hosts']:
            for host_group in host['host_groups']:
                self.config[host_group] = {}
        return True

    def add_ips_to_groups(self, inven_hosts, layout):
        # create a ip to host mapping based on count
        ip_to_host = {}
        inven_hosts.reverse()
        for host_name in layout['hosts']:
            if 'count' in host_name.keys():
                count = host_name['count']
            else:
                count = 1
            host_list = []
            if count > len(inven_hosts):
                count = len(inven_hosts)
            for i in range(count, 0, -1):
                item = inven_hosts.pop()
                host_list.append(item)
            ip_to_host[host_name["name"]] = host_list
        # add ips to the respective host groups in inventory
        for host_name in layout['hosts']:
            host_ips = ip_to_host[host_name["name"]]
            for ip in host_ips:
                for host_group in host_name['host_groups']:
                    if not ("hosts" in self.config[host_group]):
                        self.config[host_group]["hosts"] = []
                    if not ("hosts" in self.config["all"]):
                        self.config["all"]["hosts"] = []
                    self.config[host_group]["hosts"].append(ip)
                    if not (ip in self.config["all"]["hosts"]):
                        self.config["all"]["hosts"].append(ip)
        return True

    def add_common_vars(self, host_groups, layout):
        # defaults cvrs to [] when they doesnot exist
        # cvrs --> common_vars
        host_groups.append("all")
        cvrs = layout["vars"] if "vars" in layout.keys() else []
        for group in host_groups:
            items = self.config[group]["hosts"]
            if not("_meta" in self.config):
                self.config["_meta"] = {}
                self.config["_meta"]["hostvars"] = {}
            for item in items:
                for var in cvrs:
                    self.config["_meta"]["hostvars"][item] = {}
                    if cvrs[var] == "__IP__":
                        self.config["_meta"]["hostvars"][item][var] = item
                    else:
                        self.config["_meta"]["hostvars"][item][var] = cvrs[var]
        return True

    def generate_inventory(self):
        return json.dumps(self.config)



import os
import subprocess
import tempfile

from fetch import Fetch
from linchpin.exceptions import LinchpinError


class FetchGit(Fetch):

    def __init__(self, ctx, fetch_type, src, dest, cache_dir, root='',
                 root_ws='', ref=None):
        super(FetchGit, self).__init__(ctx, fetch_type, dest, root=root,
                                       root_ws=root_ws, ref=ref)
        self.src = src

        self.cache_dir = os.path.join(cache_dir, "git")
        if not os.path.exists(self.cache_dir):
            os.mkdir(self.cache_dir)


    def fetch_files(self):
        # The key cannot contain ':' since linchpin does not support python 3.
        # Configparser uses ':' as a delimiter, which poses problem when using
        # urls in key during parsing. Delimiters can be specified when
        # initializing a configparser object in python 3 so this does not
        # become an issue.

        ref = 'None'
        if self.ref:
            ref = self.ref

        key = "{0}|{1}".format(self.src.replace(':', ''), ref)

        fetch_dir = self.cfgs["git"].get(key, None)
        self.td = self.call_clone(fetch_dir)

        self.td_w_root = '{0}/{1}'.format(self.td, self.root)

        if not fetch_dir:
            self.write_cfg("git", key, self.td)


    def call_clone(self, fetch_dir=None):

        ref = None
        src = self.src
        if self.ref:
            ref = self.ref
            src = '{0}@{1}'.format(self.src, ref)

        if fetch_dir and os.path.exists(fetch_dir):
            cmd = ['git', '-C', fetch_dir, 'pull', '--quiet']
            retval = subprocess.call(cmd, stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
        else:
            if not fetch_dir:
                fetch_dir = tempfile.mkdtemp(prefix="git_", dir=self.cache_dir)

            cmd = ['git', 'clone', '--quiet', self.src]
            if ref:
                cmd.extend(['-b', ref])

            cmd.append(fetch_dir)

            retval = subprocess.call(cmd, stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)

        if retval != 0:
            raise LinchpinError("Unable to clone {0}".format(src))
        return fetch_dir



import os
import sys

from nose.tools import assert_equal
from nose.tools import assert_dict_equal
from nose.tools import assert_is_instance
from nose.tools import assert_dict_contains_subset
from nose.tools import with_setup

from linchpin import LinchpinAPI
from linchpin.utils.dataparser import DataParser
from linchpin.context import LinchpinContext
from linchpin.rundb import RunDB

from linchpin.tests.mockdata.contextdata import ContextData


def test_api_create():

    lpc = LinchpinContext()
    lpc.load_config()
    lpc.load_global_evars()
    lpa = LinchpinAPI(lpc)

    assert_equal(isinstance(lpa, LinchpinAPI), True)


def setup_load_config():

    """
    Perform setup of ContextData() object, and run get_temp_filename()
    """

    global cd
    global provider
    global config_path
    global config_data

    provider = 'dummy'

    cd = ContextData()
    cd.load_config_data()
    cd.create_config()
    config_path = cd.get_temp_filename()
    config_data = cd.cfg_data
    cd.write_config_file(config_path)


def setup_lp_api():

    """
    Perform setup of LinchpinContext, lpc.load_config, and LinchPinAPI
    """

    global lpc
    global lpa
    global pf_data
    global target
    global provision_data

    setup_load_config()

    lpc = LinchpinContext()
    lpc.load_config(search_path=[config_path])
    lpc.load_global_evars()
    lpc.setup_logging()

    lpa = LinchpinAPI(lpc)

    pinfile = lpc.get_cfg('init', 'pinfile', default='PinFile')

    base_path = '{0}'.format(os.path.dirname(
        os.path.realpath(__file__))).rstrip('/')
    mock_path = '{0}/{1}/{2}'.format(base_path, 'mockdata', provider)

    if not lpa.workspace:
        lpa.workspace = mock_path

    lpa.set_evar('workspace', mock_path)

    pf_w_path = '{0}/PinFile'.format(mock_path, pinfile)

    parser = DataParser()
    pf_d = None
    pf_data = parser.process(pf_w_path)

    topo_folder = lpc.get_evar('topologies_folder')
    topo_file = pf_data[provider]["topology"]
    topo_path = '{0}/{1}/{2}'.format(mock_path, topo_folder, topo_file)
    with open(topo_path, 'r') as topo_stream:
        topology_data = parser.parse_json_yaml(topo_stream)
        provision_data = {provider: {'topology': topology_data}}


@with_setup(setup_lp_api)
def test_setup_rundb():

    rundb = lpa.setup_rundb()
    assert_is_instance(rundb, RunDB)


@with_setup(setup_lp_api)
def test_get_cfg_section():

    lpa.set_cfg('test', 'key', 'value')
    lpa_dict = lpa.get_cfg('test')

    assert_dict_equal(lpa_dict, lpa.ctx.cfgs.get('test'))


@with_setup(setup_lp_api)
def test_get_cfg_item():

    lpa.set_cfg('test', 'key', 'value')
    lpa_value = lpa.get_cfg('test', 'key')

    test_dict = lpa.ctx.cfgs.get('test')
    test_value = test_dict.get('key')

    assert_equal(lpa_value, test_value)


@with_setup(setup_lp_api)
def test_set_cfg():

    test_dict = {'key': 'value', 'key2': 'value2'}

    for k, v in test_dict.items():
        lpa.set_cfg('test', k, v)

    assert_dict_equal(test_dict, lpa.ctx.cfgs.get('test'))


@with_setup(setup_lp_api)
def test_get_all_evars():

    test_evars = {'ekey1': 'evalue1', 'ekey2': 'evalue2'}
    for k, v in test_evars.items():
        lpa.set_evar(k, v)

    lpa_evars = lpa.get_evar()

    assert_dict_contains_subset(test_evars, lpa_evars)


@with_setup(setup_lp_api)
def test_get_evar_item():

    test_evars = {'ekey2': 'evalue2'}
    for k, v in test_evars.items():
        lpa.set_evar(k, v)

    assert_equal(lpa.get_evar(key='ekey2'), test_evars.get('ekey2'))


@with_setup(setup_lp_api)
def test_set_evar():

    test_evars = {'ekey': 'evalue'}

    for k, v in test_evars.items():
        lpa.set_evar(k, v)
    lpa_evars = lpa.get_cfg('evars')

    assert_dict_contains_subset(test_evars, lpa_evars)


@with_setup(setup_lp_api)
def test_set_hook_state():

    pass


@with_setup(setup_lp_api)
def test_bind_to_hook_state():

    pass



@with_setup(setup_lp_api)
def test_find_playbook_path():

    pb_path = lpa._find_playbook_path(provider)

    assert os.path.exists(os.path.expanduser(pb_path))


@with_setup(setup_lp_api)
def test_validate_topology():

    success = False
    topo = provision_data.get(provider).get('topology')
    try:
        resources = lpa.validate_topology(topo)
        success = len(resources)
    except Exception:
        pass

    assert success


@with_setup(setup_lp_api)
def test_do_action():

    return_code, results = lpa.do_action(provision_data)

    failed = False
    if return_code:
        failed = True
        for target, data in results.iteritems():
            task_results = data['task_results'][0]

            if not isinstance(task_results, int):
                trs = task_results
                if trs is not None:
                    trs.reverse()
                    tr = trs[0]
                    if tr.is_failed():
                        msg = tr._check_key('msg')
                        print("Target '{0}': {1} failed with"
                              " error '{2}'".format(target,
                                                    tr._task,
                                                    msg))
            else:
                if task_results:
                    return_code = task_results

    assert not failed

@with_setup(setup_lp_api)
def test_do_validation():
    return_code, results = lpa.do_validation(provision_data)

    failed = False
    if return_code:
        failed = True
        for target, data in results.iteritems():
            if not data.startswith("valid"):
                print("Validation for target '{0}': has failed with"
                      " error '{1}'".format(target, msg))

    assert not failed

@with_setup(setup_lp_api)
def test_lp_journal():

    x = lpa.lp_journal(targets=[provider])
    assert x.get(provider)


@with_setup(setup_lp_api)
def test_invoke_playbooks():

    topo = provision_data.get(provider).get('topology')
    resources = topo.get('resource_groups')

    rundb = lpa.setup_rundb()  # noqa F841
    lpa.set_evar('rundb_id', 1)

    lpa.set_evar('topo_data', topo)
    lpa.set_evar('target', provider)
    lpa.set_evar('resources', resources)
    lpa.set_evar('uhash', 'test')

    return_code, results = lpa._invoke_playbooks(resources,
                                                 action='up',
                                                 console=True)

    assert return_code == 0


def main():
    pass


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author: Samvaran Kashyap Rallabandi -  <srallaba@redhat.com>
#

import os
import yaml

from ansible.module_utils.basic import AnsibleModule

DOCUMENTATION = '''
---
version_added: "0.1"
module: output_parser
short_description: output_parser module in ansible
description:
  - This module allows a user to parse a yaml file
    and register it as variable in ansible.

options:
  output_file:
    description:
      path to topology output file can be in json/yaml format
    required: true

author: Samvaran Kashyap Rallabandi -
'''


def check_file_paths(module, *args):
    for file_path in args:
        if not os.path.exists(file_path):
            output = "File not found %s not found"
            module.fail_json(msg=output % (file_path))
        if not os.access(file_path, os.R_OK):
            output = "File not accesible %s not found"
            module.fail_json(msg=output % (file_path))
        if os.path.isdir(file_path):
            output = "Recursive directory not supported  %s "
            module.fail_json(msg=output % (file_path))


def main():
    module = AnsibleModule(
        argument_spec=dict(output_file=dict(required=True,
                                            aliases=[
                                                'topology_output_file',
                                                'resources_file'
                                            ])),
        required_one_of=[],
        supports_check_mode=True)

    data_file_path = os.path.expanduser(module.params['output_file'])

    check_file_paths(module, data_file_path)
    content = open(data_file_path, "r").read()
    c = yaml.load(content)
    resp = {"path": data_file_path, "content": c}
    if resp["content"]:
        changed = True
        module.exit_json(changed=changed, output=resp)
    else:
        module.fail_json(msg=resp)


main()



#!/usr/bin/env python

import yaml
import json
import subprocess
import yamlordereddictloader

# CentOS 6 EPEL provides an alternate Jinja2 package
try:
    from jinja2 import BaseLoader
    from jinja2 import Environment
except ImportError:
    import sys
    sys.path.insert(0, '/usr/lib/python2.6/site-packages/Jinja2-2.6-py2.6.egg')
    from jinja2 import BaseLoader
    from jinja2 import Environment


from linchpin.exceptions import LinchpinError
from linchpin.exceptions import ValidationError


class DataParser(object):


    def __init__(self):

        self._mapping_tag = yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG


    def process(self, file_w_path, data=None):
        """ Processes the PinFile and any data (if a template)
        using Jinja2. Returns json of PinFile, topology, layout,
        and hooks.

        :param file_w_path:
            Full path to the provided file to process

        :param data:
            A JSON representation of data mapped to a Jinja2 template in
            file_w_path

        """

        if not data:
            data = '{}'

        with open(file_w_path, 'r') as stream:
            file_data = stream.read()

            if data.startswith('@'):
                with open(data[1:], 'r') as strm:
                    data = strm.read()

            try:
                file_data = self.render(file_data, data, ordered=False)
                return self.parse_json_yaml(file_data)
            except TypeError:
                error_txt = "Error attempting to parse PinFile data file."
                error_txt += "\nTemplate-data files require a prepended '@'"
                error_txt += " (eg. '@/path/to/template-data.yml')"
                error_txt += "\nPerhaps the path to the PinFile or"
                error_txt += " template-data is missing or the incorrect path?."
                raise ValidationError(error_txt)

        return self.load_pinfile(file_w_path)


    def render(self, template, context, ordered=True):
        """
        Performs the rendering of template and context data using
        Jinja2.

        :param template:
            Full path to the Jinja2 template

        :param context:
            A dictionary of variables to be rendered againt the template
        """

        c = self.parse_json_yaml(context, ordered=ordered)
        t = Environment(loader=BaseLoader).from_string(template)
        return t.render(c)

    def parse_json_yaml(self, data, ordered=True):

        """ parses yaml file into json object """

        d = None

        try:
            if ordered:
                data = yaml.load(data, Loader=yamlordereddictloader.Loader)
            else:
                data = yaml.load(data)
        except Exception as e:
            raise LinchpinError('YAML parsing error: {}'.format(e))

        if isinstance(data, dict):
            return data

        return d


    def run_script(self, script):

        sp = None
        try:
            sp = subprocess.Popen(script,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)
        except OSError as e:
            raise ValidationError("problem running {0} ({1})".format(script, e))

        (stdout, stderr) = sp.communicate()

        if sp.returncode != 0:
            raise ValidationError("Script {0}"
                                  " had execution error".format(script))

        return stdout


    def load_pinfile(self, pinfile):
        # try to convert the data into json
        pf = None
        try:
            with open(pinfile, 'r') as stream:
                pf_data = stream.read()
                pf = self.parse_json_yaml(pf_data)
                # ordered hooks gives parsing errors
                # since hooks are already in lists we need not maintain order
                pf['hooks'] = self.parse_json_yaml(pf_data,
                                                   ordered=False).get('hooks',
                                                                      {})
        except ValidationError as e:
            pass

        if not pf:
            # assume not json, so dynamic script
            # once executed, verify it is json
            try:
                pf = json.loads(self.run_script(pinfile))
            except Exception as e:
                raise LinchpinError(e)

        return pf

    def write_json(self, provision_data, pf_outfile):

        with open(pf_outfile, 'w') as outfile:
            json.dump(provision_data, outfile, indent=4)
